// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const CC = Components.classes;
const CI = Components.interfaces;
const CU = Components.utils;

var gProfiler = Components.classes["@flock.com/profiler;1"]
                          .getService(Components.interfaces.flockIProfiler);
var gMBProfEvt = gProfiler.profileEventStart("mediabar");

var $ = function(aId) {
  return document.getElementById(aId);
}

// The mediaOverlay uses this to pass in an initial query or person to load.
var initialMedia;

const FLNS = "http://flock.com/rdf#";
const RECENT_ROOT = "urn:media:recent";
const FAV_CHANNELS_ROOT = "urn:media:favorites";
const PERSON_QUERY_START = "user:";
const PER_PAGE = 50;
const MAX_RECENT = 20; // The maximum length of the recent channels list
const FLOCK_DEFAULT_MEDIA_SEARCH_PREF = "flock.media.defaultquery.service";
const FLOCK_CURRENT_MEDIA_SEARCH_PREF = "flock.media.search.currentEngine";
const FLOCK_MEDIA_BAR_DEFAULT_MEDIA_STREAMS_IGNORED_SRVCS_LIST_PREF = "flock.mediabar.defaultMediaStreams.ignoredServicesList";
const FLOCK_MEDIA_BAR_SEARCHBAR_IGNORED_SRVCS_LIST_PREF = "flock.mediabar.searchBar.ignoredServicesList";

const FLOCK_MEDIA_ADD_FAV_DLG_PREF = "flock.media.addFavDlg.show";
const FLOCK_PARTNER_EDITION_PREF = "general.useragent.edition";
// delay time before loading previous stream after "Clear all" action
const CLEAR_NEW_LOAD_DELAY = 100;

const height_small = 144;
const height_small_meta = 147;
const height_large = 250;

var gPhotoDrawer = null;
var gInitialActionMessage = null;

var photoUploadSvc = Components.classes["@mozilla.org/rdf/datasource;1?name=flock-photo-uploads"]
                               .getService(Components.interfaces.flockIPhotoUploadService);

// The photo drawer controller

var PhotoDrawer = function(aNode, aSlideybox) {
  this.mSlideybox = aSlideybox;
  this.mGroupbarBox = this.mSlideybox.groupbarBox;
  this.pref_service = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefBranch);
  this.acUtils = Components.classes["@flock.com/account-utils;1"]
                           .getService(Components.interfaces.flockIAccountUtils);
  this.metrics = Components.classes["@flock.com/metrics-service;1"]
                           .getService(Components.interfaces.flockIMetricsService);
  this.mediaObserverService = Components.classes["@mozilla.org/observer-service;1"]
                                  .getService( Components.interfaces.nsIObserverService );
  this.logger = Components.classes["@flock.com/logger;1"].createInstance(Components.interfaces.flockILogger);
  this.logger.init("mediabar");
  this.mFinishedTasks = 0;

  // mLastSearchRequestId is a timestamp value used to track the most recent
  // search request (which should be the only request processed by the
  // response handler)
  this.mLastSearchRequestId = new Date().getTime();
  this.init();
  this.loadInitialMedia();
  this.mediaObserver = { 
    observe: function mediaObserver_observe(aSubject, aTopic, aData) {
      if (aTopic == "mediaFinished") {
        // There are 2 tasks we are relying on: searchPerson() and loadQuery()
        if (gPhotoDrawer.mFinishedTasks == 2 && gPhotoDrawer.isEmpty) {
          if (gPhotoDrawer.hasPerson) {
            var message = flockGetString("photo/photo", "flock.media.personfound.nomedia.label");
            gPhotoDrawer.showActionMessage(message);
          } else {
            var message = flockGetString("photo/photo", "flock.media.nomedia.label");
            gPhotoDrawer.showActionMessage(message);
          }
        }
      }
    }
  };
  this.mediaObserverService
      .addObserver(this.mediaObserver, "mediaFinished", false);
  this.apiAuthObserver = {
    observe: function apiAuthObserver_observe(aSubject, aTopic, aData) {
      if (aTopic == "apiAuthChange") {
        // Mediabar has detected that a service api auth has changed. Refresh
        // the mediabar only if the current stream is from the same service. We
        // need to do this to show/hide private media depending on the api auth
        // state.
        var svcShortname = aData;
        if (svcShortname == gPhotoDrawer.mApi) {
          gPhotoDrawer.refreshView();
        }
      }
    } 
  };
  this.mediaObserverService
      .addObserver(this.apiAuthObserver, "apiAuthChange", false);
}

PhotoDrawer.prototype = {
  init: function pdInit() {
    this.metrics.report("MediaBar-Opened", null);

    var inst = this;
    this.mFinishedTasks = 0;
    this.mDnDInProgress = false;
    // Status of the UI
    this.mFilter = "";
    this.mPage = 0;
    this.isEmpty = true;
    this.isPersonQuery = false;
    this.objSynch = false;
    this.mControlBoxList = [];
    this.hasPerson = false;

    // Load engines based on RSS feeds from the pref
    var rssSearchJSON = this.pref_service.getCharPref("flock.photo.rssSearch");
    var s = new CU.Sandbox("about:blank");
    this.mRSSEngines = CU.evalInSandbox(rssSearchJSON, s)[0];
    this.lastRSSEngine = null;

    // persist which photo size was last selected
    if ($("mediabar").getAttribute("photoSizeLarge") == "true") {
      this.setPhotoSize("large");
    } else {
      this.setPhotoSize("small");
      if (this.getMode() == "consume") {
        top.flock_topbarSize(height_small);
      } else {
        top.flock_topbarSize(height_small_meta);
      }
    }

    // hide group bar in consume mode
    if (this.getMode() == "consume") {
      this.mGroupbarBox.setAttribute("hidden", "true");
    }

    // Information about the current channel
    this.mApi = null;
    this.mQuery = new queryHelper();
    this.mName = "";
    this.mAlbumLabel = "";
    this.mAlreadyLoaded = false;
    this.mPerson = null;
    this.mChannel = null;
    this.mHasPrivateMedia = false;

    this.mRDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                                 .getService(Components.interfaces.nsIRDFService);
    this.mFavsDS = this.mRDFService.GetDataSource("rdf:flock-favorites");
    this.mFavsDS.QueryInterface (Components.interfaces.flockIRDFObservable);

    this.unseenItemsPredicate = this.mRDFService.GetResource(FLNS + "hasUnseenItems");
    this.mFavsDS.addArcObserver(Components.interfaces.flockIRDFObserver.TYPE_CHANGE,
                                null, this.unseenItemsPredicate, null,
                                unseenItemsObs);

    // Coop stuff
    this._coop = Components.classes["@flock.com/singleton;1"]
                           .getService(Components.interfaces.flockISingleton)
                           .getSingleton("chrome://flock/content/common/load-faves-coop.js")
                           .wrappedJSObject;
    this.recentRoot = this._coop.get(RECENT_ROOT);
    if (!this.recentRoot) {
      this.recentRoot = new this._coop.Folder(RECENT_ROOT);
      this._coop.favorites_root.children.add(this.recentRoot);
    }
    this.favChannelsRoot = this._coop.get(FAV_CHANNELS_ROOT);
    if (!this.favChannelsRoot ) {
      this.favChannelsRoot = new this._coop.Folder(FAV_CHANNELS_ROOT);
      this._coop.favorites_root.children.add(this.favChannelsRoot);
    }

    // UI stuff
    this.slideyBoxListener = {
      onDrop: function onDrop(aSelection, aHoverChild) {
      },
      onScrollLeft: function onScrollLeft() {
      },
      onScrollRight: function onScrollRight() {
        var pagesLeft = inst.mSlideybox.pagesLeft;
        if (pagesLeft < 3.3) {
          gPhotoDrawer.loadMore();
        }
      },
      onRemoveSelectionLeft: function onRemoveSelectionLeft() {
      },
      onRemoveSelection: function onRemoveSelection(aSelection) {
      }
    }

    this.mSlideybox.setListener(this.slideyBoxListener);

    // add the list of default channels to the popup and populate the search dropdown
    var channelsPopup = $("mediabar-channels-popup");
    var channelsSeperator = $("mediabar-featured-seperator")
    var searchPopup = $("mediabar-search");

    this.mApiMgr = Components.classes["@flock.com/photo-api-manager;1?"]
                             .getService(Components.interfaces.flockIPhotoAPIManager);
    var svcs = this.mApiMgr.services;
    
    // Get the list of search and media streams items that will be ignored 
    var prefIgnoredMediaStreamsList = ""; 
    this.pref_service.getPrefType(FLOCK_MEDIA_BAR_DEFAULT_MEDIA_STREAMS_IGNORED_SRVCS_LIST_PREF) && (prefIgnoredMediaStreamsList = 
    this.pref_service.getCharPref(FLOCK_MEDIA_BAR_DEFAULT_MEDIA_STREAMS_IGNORED_SRVCS_LIST_PREF));
    
    var prefIgnoredSearchServicesList = "";
    this.pref_service.getPrefType(FLOCK_MEDIA_BAR_SEARCHBAR_IGNORED_SRVCS_LIST_PREF) && (prefIgnoredSearchServicesList = 
    this.pref_service.getCharPref(FLOCK_MEDIA_BAR_SEARCHBAR_IGNORED_SRVCS_LIST_PREF));

    prefIgnoredSearchServicesList += ",mediarss";

    // Add the Media RSS search engines.
    for (var key in this.mRSSEngines) {
      var engine = this.mRSSEngines[key];
      searchmenuitem = document.createElement("menuitem");
      searchmenuitem.setAttribute("label",
                                  flockGetString("photo/mediabar",
                                                 "flock.media.search.title",
                                                 [engine.title]));
      searchmenuitem.setAttribute("image", engine.icon);
      searchmenuitem.setAttribute("class", "menuitem-iconic");
      searchmenuitem.setAttribute("engine", key);
      searchmenuitem.setAttribute("oncommand",
                                  "gPhotoDrawer.searchRSS(this.getAttribute('engine'));"
                                  + "gPhotoDrawer.searchbarFocus();");
      $("media-search-menupopup").appendChild(searchmenuitem);
    }

    // Iterate through the list of services and populate:
    // 1) media streams list
    // 2) search bar list

    function populateServiceChannel(aService,
                                    aChannel,
                                    aParentChannelArray,
                                    aDOMNode)
    {
      // Iterate through subchannels and append output
      if (aChannel.subchannels) {
        // Create menu for channel to hold it's children
        var menu = document.createElement("menu");
        menu.setAttribute("label", aChannel.title);
        aDOMNode.appendChild(menu);
        var menupopupsub = document.createElement("menupopup");
        menu.appendChild(menupopupsub);
        aParentChannelArray.push(aChannel);
        while (aChannel.subchannels.hasMoreElements()) {
          var subchannel = aChannel.subchannels.getNext();
          subchannel.QueryInterface(CI.flockIMediaChannel);
          // Recursively check for sub channels
          populateServiceChannel(aService,
                                 subchannel,
                                 aParentChannelArray,
                                 menupopupsub);
        }
      } else {
        var channelmenuitem = document.createElement("menuitem");
        var title = aChannel.title;
        if (aParentChannelArray.length) { 
          for (var k = aParentChannelArray.length - 1; k >= 0; k--) {
            title = aParentChannelArray[k].title + " - " + title;
          }
        }
        channelmenuitem.setAttribute("label", aChannel.title);
        channelmenuitem.setAttribute("value", aChannel.id);
        var oncommand =
          "gPhotoDrawer.loadServiceChannel('" + aService.shortName + "', "
                                              + "'" + aChannel.id + "', "
                                              + "'" + title + "');";
        channelmenuitem.setAttribute("oncommand", oncommand);
        aDOMNode.appendChild(channelmenuitem);
      }
    }

    while (svcs.hasMoreElements()) {
      var svc = svcs.getNext();
      
      // Add the service to the media streams list if it is not on the ignore list (configured for branding)
      if (prefIgnoredMediaStreamsList.split(',').indexOf(svc.shortName) == -1) { 
        var menu = document.createElement("menu");        
        menu.setAttribute("label", svc.title);
        menu.setAttribute("image", svc.icon);
        menu.setAttribute("class", "menu-iconic");
        var menupopup = document.createElement("menupopup");
        menu.appendChild(menupopup);

        var channels = svc.enumerateChannels();
        var count = 0;
        if (channels && channels.hasMoreElements()) {
          while (channels.hasMoreElements()) {
            var channel = channels.getNext();
            channel.QueryInterface(CI.flockIMediaChannel);
            populateServiceChannel(svc, channel, [], menupopup);
            count++;
          }
        }
    
        if (count > 0) {
          channelsPopup.insertBefore(menu, channelsSeperator);
        }
      }
      
      // Add the service to the media bar search bar if it is not on the ignore list (configured for branding)
      if (prefIgnoredSearchServicesList.split(',').indexOf(svc.shortName) == -1) { 
        var searchmenuitem = document.createElement("menuitem");
        searchmenuitem.setAttribute("label", flockGetString("photo/mediabar", "flock.media.search.title", [svc.title]));
        searchmenuitem.setAttribute("image", svc.icon);
        searchmenuitem.setAttribute("class", "menuitem-iconic");
        searchmenuitem.setAttribute("oncommand", "gPhotoDrawer.searchService('" +
                                    svc.shortName +
                                    "'); gPhotoDrawer.searchbarFocus();");
        $("media-search-menupopup").appendChild(searchmenuitem); 
      }
    }

    var searchSvc = this.pref_service
                        .getCharPref(FLOCK_DEFAULT_MEDIA_SEARCH_PREF);
    if (this.pref_service.getPrefType(FLOCK_CURRENT_MEDIA_SEARCH_PREF)) {
      searchSvc = this.pref_service
                      .getCharPref(FLOCK_CURRENT_MEDIA_SEARCH_PREF);
    }

    if (searchSvc.substring(0,8) == "mediarss") {
      this.searchRSS(searchSvc.split(":")[1]);
    } else {
      this.searchService(searchSvc);
    }

    this.updateFavoritesChannelUI();

    this.mediabarForwardButton = $("mediabar-stream-forward");
    this.mediabarBackButton = $("mediabar-stream-back");

    // if there are no media accounts configured, pop message bar
    var mediaAccts = this.acUtils.getAccountsByInterface("flockIMediaWebService");
    if (!mediaAccts.hasMoreElements()) {
      var message = flockGetString("photo/photo", "flock.photo.account.discovery.label");
      var nBox = top.gBrowser.getNotificationBox();
      var links = [{
        label: flockGetString("photo/photo", "flock.photo.account.discovery.link"),
        callback: function() {
          toggleSidebarWithMetric("flock_AccountsSidebarBroadcaster",
                                  "AccountSidebar-Open",
                                  null,
                                  "MediabarPhotoDiscoveryNoAccount",
                                  true);
          var obs = Components.classes["@mozilla.org/observer-service;1"]
                              .getService(Components.interfaces.nsIObserverService);
          setTimeout(function asbar_timeout() {
            obs.notifyObservers(null, "toggleAccountsArea", "media,people");
          }, 300);
        }
      }];
      nBox.appendUniqueNotification(message,
                                    "account-needed",
                                    null,
                                    nBox.FLOCK_PRIORITY_LOW,
                                    null, links, "accounts");
    }
    gProfiler.profileEventEnd(gMBProfEvt, "");
  },

  uninit: function pdUninit() {
    this.metrics.report("MediaBar-Close", null);

    this.clear();

    // var unseenItemsPredicate = this.mRDFService.GetResource(FLNS + "hasUnseenItems");
    if (updateTimer) {
      clearTimeout(updateTimer);
    }
    updateTimer = null;
    this.mFavsDS.removeArcObserver(Components.interfaces.flockIRDFObserver.TYPE_CHANGE,
                                   null, this.unseenItemsPredicate, null,
                                   unseenItemsObs);
   this.mediaObserverService.removeObserver(this.mediaObserver, 'mediaFinished' );
   this.mediaObserverService.removeObserver(this.apiAuthObserver,
                                            "apiAuthChange");
    // unseenItemsObs.rdfChanged = null;
    // unseenItemsObs = null;
  },

  loadInitialMedia: function pdLoadInitialMedia() {
    // if there is new content, set initial media to be Preview New stream
    if (!initialMedia && this.mApiMgr.hasNewMedia) {
      initialMedia = {
        query: {
          service: "preview",
          query: "preview",
          name: flockGetString("photo/mediabar", "flock.media.search.prev.new")
        }
      }
    }

    if (initialMedia && initialMedia.query) {
      this.loadQuery(initialMedia.query.service,
                     initialMedia.query.query,
                     initialMedia.query.name);
      if (this.recentRoot.children.enumerate().hasMoreElements()) {
        this._disableMediabarBackButton(false);
        this._disableMediabarForwardButton(true);
      }
      return;
    }

    if (initialMedia && initialMedia.person) {
      this.loadPhotoPerson(initialMedia.person.urn, initialMedia.person.svc);
      if (this.recentRoot.children.enumerate().hasMoreElements()) {
        this._disableMediabarBackButton(false);
        this._disableMediabarForwardButton(true);
      }
      return;
    }

    var nonPreviewStreamFound = false;
    var theEnum = this.recentRoot.children.enumerate();
    // look for the first non-preview stream in recent history to load
    while (theEnum.hasMoreElements()) {
      var recentItem = theEnum.getNext();
      if (recentItem.service != "preview") {
        nonPreviewStreamFound = true;
        this.loadFromId(recentItem.id());
        break;
      }
    }

    // if there are no non-preview streams in recent history OR
    // if recent history is empty, load default media stream
    if (!nonPreviewStreamFound) {
      var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService)
                            .getBranch("flock.media.defaultquery.");
      var service = prefs.getCharPref("service");
      var query = prefs.getCharPref("query");
      var label = prefs.getCharPref("label");
      this.loadQuery(service, query, label);
    }
    this.onSearchboxBlur();
    this._disableMediabarBackButton(!theEnum.hasMoreElements());
    this._disableMediabarForwardButton(true);
  },

  loadServiceChannel: function pdLoadServiceChannel(aServiceShortName,
                                                    aValue,
                                                    aLabel)
  {
    this.mFilter = "";
    this.loadQuery(aServiceShortName, aValue, aLabel);
    this.emptySearchbox();
    this.metrics.report("MediaBar-SelectChannel", [{service: aServiceShortName,
                                                    channel: aValue}]);
  },

  searchPerson: function pdSearchPerson(svcShortname, queryString) {
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(svcShortname);
    if (svc instanceof Components.interfaces.flockIMediaWebService) {
      var str = queryString;
      if (!str && !$("mediabar-search-textbox").hasAttribute("empty")) {
        str = $("mediabar-search-textbox").value;
      }
      if (!str || str == "") {
        return;
      }
      var inst = this;
      var findUserListener = {
        onStart: function pd_fbu_onStart(aSubject, aTopic) {
        },
        onSuccess: function pd_fbu_onSuccess(aSubject, aTopic) {
          aSubject.QueryInterface(CI.flockIPerson);
          inst.hasPerson = true;
          var query = new queryHelper();
          query.user = aSubject.accountId;
          query.username = str;
          var name = str;
          inst.clearSnowman();
          inst.showSnowman(aSubject, svcShortname, query, name);
          inst.mFinishedTasks++;
          inst.mediaObserverService.notifyObservers(null, 'mediaFinished', null);
        },
        onError: function pd_fbu_onError(aSubject, aTopic, aError) {
         inst.mFinishedTasks++;
         inst.mediaObserverService.notifyObservers(null, 'mediaFinished', null);
        }
      };
      this.logger.debug("Asking " + svc.shortName + " for user '" + str + "'");
      svc.findByUsername(findUserListener, str);
    } else {
      this.logger.debug("svc does not implement flockIMediaWebService"
                        + " - skipping searchPerson()");
      this.mFinishedTasks++;
      this.mediaObserverService.notifyObservers(null, 'mediaFinished', null);
    }
  },

  showSnowman: function pdShowSnowman(aPerson, svcShortname, query, name) {
    var SNOWMAN_ICON = "chrome://browser/skin/flock/services/common/no_avatar.png";
    if (aPerson.avatar) {
      SNOWMAN_ICON = aPerson.avatar;
    }
    var snowmanLabel = document.createElement("label");
    snowmanLabel.setAttribute("value", flockGetString("photo/mediabar", "flock.media.search.person.found"));
    var snowman = document.createElement("image");
    snowman.setAttribute("src", SNOWMAN_ICON);
    snowman.setAttribute("style", "cursor: pointer;");
    snowman.setAttribute("query", query.stringVal);
    snowman.setAttribute("username", name);
    snowman.setAttribute("onclick", "gPhotoDrawer.loadQuery( '" +
                                    svcShortname + "', this.getAttribute('query'), this.getAttribute('username'));" +
                                    "gPhotoDrawer.metrics"
                                    + ".report('MediaBar-UserClick', '" 
                                    + svcShortname + "');");
    snowman.setAttribute("tooltiptext", flockGetString("photo/mediabar", "flock.media.search.person.view", [aPerson.name]));
    var label = document.createElement("label");
    label.setAttribute("value", aPerson.name);
    label.setAttribute("crop", "end");
    label.setAttribute("style", "font-weight: bold; max-width: 75px; cursor: pointer;");
    label.setAttribute("query", query.stringVal);
    label.setAttribute("username", name);
    label.setAttribute("onclick", "gPhotoDrawer.loadQuery('" + svcShortname +
                                  "', this.getAttribute('query'), this.getAttribute('username'));");
    label.setAttribute("tooltiptext", flockGetString("photo/mediabar", "flock.media.search.person.view", [aPerson.name]));
    $("snowman").appendChild(snowmanLabel);
    $("snowman").appendChild(snowman);
    $("snowman").appendChild(label);
    // display user photo count. see flockBug 7105
    if (aPerson.totalMedia > -1) {
      var photoCountLabel = document.createElement("label");
      photoCountLabel.setAttribute("value",
                                   flockGetString("photo/mediabar",
                                                  "flock.media.search.person.number",
                                                   [aPerson.totalMedia]));
      $("snowman").appendChild(photoCountLabel);
    }
    $("snowman").setAttribute("empty", "false");
  },

  clearSnowman: function pdClearSnowman() {
    $("snowman").setAttribute("empty", "true");
    while ($("snowman").lastChild) {
      $("snowman").removeChild($("snowman").lastChild);
    }
  },

  close: function pdClose() {
    top.flock_topbarHide();
    this.clear();
  },

  move: function pdMove(ignoreLocation) {
    var location = top.flock_topbarMove(ignoreLocation);
    $("mediabar").setAttribute("location", location);
  },

  // Recursively removes children from a parent in the DOM
  recursiveClear: function pdRecursiveClear(aParent) {
    if (aParent) {
      while (aParent.hasChildNodes()) {
        var child = aParent.lastChild;
        if (child) {
          this.recursiveClear(child);
          aParent.removeChild(child);
        }
      }
    }
  },

  clear: function pdClear() {
    // reclaim some system memory by removing DOM nodes from the slideybox (mediabar)
    this.recursiveClear(this.mSlideybox);
    this.recursiveClear(this.mGroupbarBox);

    this.mFinishedTasks = 0;
    this.mPage = 0;
    this.isEmpty = true;
    this.hasPerson = false;
    this.mHasPrivateMedia = false;
    this.mAlreadyLoaded = false;
    this.mControlBoxList = [];
    this.mSlideybox.scrollHome();
    this.clearSnowman();
  },

  refreshView: function pdRefreshView() {
    this.addRecent();
    this.loadFirst();
    if (this.mQuery.search) {
      this.searchPerson(this.mApi, this.mQuery.search);
    }
  },

  createGroupBar: function pdCreateGroupBar(aSecondPhoto, aFirstPhoto) {
    var newBar = document.createElement("hbox");
    newBar.id = "mediabar-groupbar-" + aFirstPhoto.id;
    newBar.setAttribute("hidden", true);

    this.mGroupbarBox.appendChild(newBar);
    aFirstPhoto.setAttribute("groupbar", newBar.id);
    if (aSecondPhoto) aSecondPhoto.setAttribute("groupbar", newBar.id);
    newBar.mSize = this.mSize;
    newBar.setAttribute("align", "center");
    newBar.setAttribute("class", "groupbar");

    var newLabel = document.createElement("label");
    newLabel.setAttribute("flex", "1");

    if (this.groupMethod(aSecondPhoto, aFirstPhoto) == "user") {
      newLabel.setAttribute("value", aFirstPhoto.username);
      newLabel.setAttribute("class", "authorLink");
      newLabel.setAttribute("querystring", "user:" + aFirstPhoto.userid + "|username:" + aFirstPhoto.username);
      newLabel.setAttribute("username", aFirstPhoto.username);
      newLabel.setAttribute("onclick",
                            "gPhotoDrawer.loadQuery('"
                                                    + aFirstPhoto.svc
                                                    +"', "
                                                    + "this.getAttribute('querystring'),"
                                                    + "this.getAttribute('username')), "
                                                    + "gPhotoDrawer.popMessageBar(this.getAttribute('querystring'));");
    } else {
      var formatter = Components.classes["@flock.com/date-formatter;1"]
                                .getService(Components.interfaces.flockIDateFormatter);
      newLabel.setAttribute("value",
                            formatter.getShortDateTimeString(new Date(Number(aFirstPhoto.mPhoto.uploadDate))));
    }
    newBar.appendChild(newLabel);
    newBar.photoCount = (aSecondPhoto) ? 2 : 1;
    newBar.setupBar = function setupBar() {
      this.removeAttribute("hidden");
    };
    newBar.changeSize = function changeSize(sizeDiff) {
      var newWidth = Number(this.getAttribute("photowidth")) + sizeDiff;
      this.setAttribute("photowidth", newWidth);
      if (newWidth < 0) {
        newWidth = 0;
      }
      this.setAttribute("width", newWidth);
      this.setAttribute("style", "width: "+this.getAttribute("width")+"px;");
    };

    this.processPhotoSize(aFirstPhoto);
    if (aSecondPhoto) {
      this.processPhotoSize(aSecondPhoto);
    }
  },

  extendGroupBar: function pdExtendGroupBar(aGroupBar, aPhoto) {
    aPhoto.setAttribute("groupbar", aGroupBar.id);
    aGroupBar.photoCount = aGroupBar.photoCount + 1;
    this.processPhotoSize(aPhoto);
  },

  groupMethod: function pdGroupMethod(secondPhoto, firstPhoto) {
    if (!this.isPersonQuery &&
        (!secondPhoto || secondPhoto.userid == firstPhoto.userid))
    {
      return "user";
    }

    // Photos are uploaded within x minutes of each other...
    if (!secondPhoto ||
        (Number(firstPhoto.mPhoto.uploadDate) - Number(secondPhoto.mPhoto.uploadDate)) < 300000)
    {
      return "date";
    }
    return;
  },

  shouldGroup: function pdShouldGroup(secondPhoto, firstPhoto) {
    if (this.groupMethod(secondPhoto, firstPhoto)) {
      return true;
    }
    return false;
  },

  groupBarAround: function pdGroupBarAround(secondPhoto, firstPhoto) {
    if (firstPhoto.getAttribute("groupbar")) {
      this.extendGroupBar($(firstPhoto.getAttribute("groupbar")), secondPhoto);
    } else {
      this.createGroupBar(secondPhoto, firstPhoto);
    }
  },

  appendMediaItem: function pdAppendMediaItem(aMediaItem) {
    var newPhoto = document.createElement("photo");
    newPhoto.setAttribute("photoControlBoxOpen", "false");
    this.mSlideybox.appendChild(newPhoto);
    if(!this.mAlreadyLoaded){
      newPhoto.setAttribute("style", "-moz-opacity: 0;");
      newPhoto.setAttribute("onload",
                            "gPhotoDrawer.fadeInPhoto(this); event.stopPropagation();");
    }
    else {
      newPhoto.setAttribute("onload", "event.stopPropagation();");
    }

    newPhoto.photo = aMediaItem;
    // Calling this before setting the media item created a leak. Weird!
    newPhoto.size = this.mSize;

    var prevPhoto = newPhoto.previousSibling;
    if (prevPhoto && this.shouldGroup(newPhoto, prevPhoto)) {
      this.groupBarAround(newPhoto, prevPhoto);
    } else if (prevPhoto && !(prevPhoto.getAttribute("groupbar"))) {
      this.createGroupBar(null, prevPhoto);
    }

    if (newPhoto.is_public == "false") {
      this.mHasPrivateMedia = true;
    }
    newPhoto.tooltipId = "mediabar-tooltip";
    if (this.mQuery.hasKey("user")) {
      newPhoto.is_single = "true";
    } else {
      // JMC - Evil special case for the preview view
      if (this.mApi == "preview") {
       newPhoto.svc = aMediaItem.svcShortName;
      } else {
        newPhoto.svc = this.mApi; // this is a string
      }
    }
    newPhoto.setupMetaData();
    this.isEmpty = false;
  },

  fadeInPhoto: function pdFadeInPhoto(aPhoto) {
    this.processPhotoSize(aPhoto);
    gDomFx.fadeIn(aPhoto, null);
  },

  loadFirst: function pdLoadFirst(aForce) {
    this.clear();
    this.setupSearch();
    this.setupAlbumMenu();
    this.loadMore();
  },

  // flockIListener#onStart
  onStart: function pdonStart(aSubject, aTopic) {
  },

  // flockIListener#onError
  onError: function pdOnError(aSubject, aRequestId, aError) {

    // Each search request is sent out with a request id (timestamp value)
    // Since we only care about the most recent request we can ignore
    // a response where the request id is older than the most recent one
    // mLastSearchRequestId represents the most recent request id
    if ((parseInt(aRequestId)) < (parseInt(this.mLastSearchRequestId))) {
      return;
    }

    document.getElementById("mediabar-statusbox").error(aError.errorString);
    // If this is a "not logged in" error,
    // send the user to the API's login page
    if (aError.errorCode == CI.flockIError.PHOTOSERVICE_USER_NOT_LOGGED_IN) {
      var svcCoopObj = this._coop.get("urn:" + this.mApi + ":service");
      var webSvc = Components.classes[svcCoopObj.serviceId]
                             .getService(Components.interfaces.flockIWebService);
      // Open in new tab and switch to it immediately
      openUILinkIn(webSvc.url, "tab");
    }
  },

  // flockIListener#onSuccess
  onSuccess: function pdonSuccess(aSubject, aRequestId) {

    // Each search request is sent out with a request id (timestamp value)
    // Since we only care about the most recent request we can ignore
    // a response where the request id is older than the most recent one
    // mLastSearchRequestId represents the most recent request id 
    if ((parseInt(aRequestId)) < (parseInt(this.mLastSearchRequestId))) {
      return;
    }

    document.getElementById("mediabar-statusbox").hidden = true;
    if (!this.mFilter) {
        this.onSearchboxBlur();
    }
    var mediaItem;

    function enumWrapper(aEnum) {
      this.innerEnum = [];
      while (aEnum.hasMoreElements()) {
        this.innerEnum.push(aEnum.getNext());
      }
      this.enumIndex = 0;
    }
    enumWrapper.prototype.rewind = function rewind(){
      this.enumIndex = 0;
    }
    enumWrapper.prototype.hasMoreElements = function hasMoreElements() {
      return this.innerEnum.length > this.enumIndex;
    }
    enumWrapper.prototype.getNext = function getNext() {
      return this.innerEnum[this.enumIndex++];
    }

    aSubject.QueryInterface(CI.nsISimpleEnumerator);
    var aEnum = new enumWrapper(aSubject);
    if (aEnum.innerEnum.length == 0) {
      this.mPage = -1; // no more photos available
    }

    while (aEnum.hasMoreElements()) {
      mediaItem = aEnum.getNext().QueryInterface(CI.flockIMediaItem);
      this.appendMediaItem(mediaItem);
    }
    if (mediaItem && $(mediaItem.id) && $(mediaItem.id).getAttribute("groupbar")) {
      $($(mediaItem.id).getAttribute("groupbar")).changeSize(-2);
      if ($(mediaItem.id).previousSibling &&
          $($(mediaItem.id).previousSibling.getAttribute("groupbar")))
      {
        $($(mediaItem.id).previousSibling.getAttribute("groupbar")).setupBar();
      }
    }

    if (this.isPersonQuery && this.isEmpty) {
      this.hasPerson = true;
      this.isEmpty = false;
      var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                          .getService(Components.interfaces.flockIPhotoAPIManager)
                          .getAPIFromShortname(this.mApi);
      var message = flockGetString("photo/photo", "flock.media.nomedia.account.label", [svc.title]); 
      this.showActionMessage(message);
      !this.bHistoryNav && this.addRecent();
      this.updateWatch();
      if (!$("mediabar").getAttribute("newMediaPreview")) {
        this.disableWatch(false);
      }
      this.mAlreadyLoaded = true;
    }else if(!this.mAlreadyLoaded) {
      !this.bHistoryNav && this.addRecent();
      this.updateWatch();
      if (!$("mediabar").getAttribute("newMediaPreview")) {
        this.disableWatch(false);
      }
      this.mAlreadyLoaded = true;
      // JMC - Only process photostream if it's the first page of results
      aEnum.rewind();
      var photoAPIManager = Components.classes["@flock.com/photo-api-manager;1?"]
                          .getService(Components.interfaces.flockIPhotoAPIManager);
      var aStreamUrn = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" +
                       this.mQuery.canonicalVal;
      photoAPIManager.processPhotoStream(aStreamUrn, aEnum, false);
    }
    this.mFinishedTasks++;
    this.mediaObserverService.notifyObservers(null, 'mediaFinished', null);
  },

  loadMore: function pdLoadMore() {
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(this.mApi);
    var query = new queryHelper(this.mQuery.stringVal);
    if (this.mFilter != "") {
      query.search = this.mFilter;
    }

    if (this.mPage == -1 ) {
      return; // no more photos available
    }
    this.mPage++;
    // Tag this request as the most recent one
    this.mLastSearchRequestId = new Date().getTime();
    svc.search(this,
               query.stringVal,
               PER_PAGE,
               this.mPage,
               this.mLastSearchRequestId);
  },

  searchService: function pdSearchService(svcShortname) {
    this.metrics.report("MediaBar-SearchEngineChange", svcShortname);
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                          .getService(Components.interfaces.flockIPhotoAPIManager)
                          .getAPIFromShortname( svcShortname );
    this.activeSearchSVC = svc;
    if (!this.activeSearchSVC) {
      svc = { svcShortname: "filter",
              icon: "chrome://flock/skin/common/lookingGlass.png"
      };
    }

    this.pref_service.setCharPref(FLOCK_CURRENT_MEDIA_SEARCH_PREF, svcShortname);
    $("mediabar-search-button").setAttribute("style",
                                             "list-style-image: url(" + svc.icon+ ");");
  },

  searchRSS: function pdSearchRSS(aEngineId) {
    this.lastRSSEngine = this.mRSSEngines[aEngineId];
    this.metrics.report("MediaBar-SearchEngineChange", "rssmedia");
    this.activeSearchSVC = CC["@flock.com/?photo-rss-media;1"]
                           .getService(CI.flockIMediaWebService);

    this.pref_service.setCharPref(FLOCK_CURRENT_MEDIA_SEARCH_PREF, "mediarss:" + aEngineId);
    var style = "list-style-image: url(" + this.lastRSSEngine.icon + ");";
    $("mediabar-search-button").setAttribute("style", style);
  },

  searchbarFocus: function pdSearchbarFocus() {
    $("mediabar-search-textbox").focus();
    $("mediabar-search-textbox").select();
  },

  performSearch: function pdPerformSearch() {
    $("mediabar-search-textbox").blur();

    var str = $("mediabar-search-textbox").value;
    if ($("mediabar-search-textbox").hasAttribute("empty")) {
      str = "";
    }

    if (!this.activeSearchSVC) {
      if (this.supportsSearch) {
        this.addRecent();
        this.mFilter = str;
        this.mQuery.search = this.mFilter;
        //this.refreshView();
        this.loadFirst();
        return;
      } else {
        this.searchService( this.mApi );
      }
    }
    if (str == "") {
      return;
    }
    var svcShortname = this.activeSearchSVC.shortName;
    this.metrics.report("MediaBar-PerformSearch", svcShortname);
    var query;
    if (svcShortname == "mediarss") {
      var url = this.lastRSSEngine.url.replace("%s", encodeURIComponent(str));
      query = "special:" + encodeURIComponent(url);
      this.loadQuery(svcShortname, query, '"' + str + '"');
    } else {
      query = new queryHelper();
      query.search = str; // TODO XXX: perhaps we should strip this?
      var name = '"' + str + '"';
      var inst = this;
      setTimeout(function pdPerformSearch_timeoutfunc() {
                   inst.searchPerson(svcShortname, str);
                 },
                 300);
      this.loadQuery(svcShortname, query.stringVal, name);
    }
  },

  keyup: function pdKeyUp(aEvent) {
    switch (aEvent.keyCode) {
      case aEvent.DOM_VK_RETURN:
        this.performSearch();
        break;

      case aEvent.DOM_VK_DOWN:
        $("media-search-menupopup").showPopup($("media-search-menupopup"),
                                              -1, -1, "popup", "bottomleft",
                                              "topleft");
        break;
    }
  },

  keydown: function pdKeyDown(aEvent) {
    // user entering text, update to proper state
    this.onSearchboxFocus();
  },

  loadAuthPerson: function pdLoadAuthPerson(aService) {
    var coopService = this._coop.get(aService.urn);
    var accounts = this._coop.Account.find({service: coopService});
    if (accounts.length > 0) {
      this.loadPhotoPerson(accounts[0].id(), aService.shortName);
    }
  },

  loadPhotoPerson: function pdLoadPhotoPerson(aPersonURN, aService) {
    this.mFilter = "";
    var identityTarget = this._coop.get(aPersonURN);
    var query = new queryHelper();
    query.user = identityTarget.accountId;
    query.username = identityTarget.name;
    this.loadQuery(aService, query.stringVal, identityTarget.name);
  },

  checkFavoritesMenu: function pdCheckFavoritesMenu() {
    var lastChild = $("flock-favorites-channels-dropdown").lastChild;
    if (!(lastChild.localName == "menuitem")) {
      var channelmenuitem = document.createElement("menuitem");
      channelmenuitem.setAttribute("label", "Empty");
      channelmenuitem.setAttribute("disabled", true)
      $("flock-favorites-channels-dropdown").appendChild(channelmenuitem);
    }
  },

  clearEmptyFavoritesMenu: function pdClearEmptyFavoritesMenu() {
    var lastChild = $("flock-favorites-channels-dropdown").lastChild;
    if(lastChild.label == "Empty") {
      $("flock-favorites-channels-dropdown").removeChild(lastChild);
    }
  },

  populateAccountsMenu: function pdPopulateAccountsMenu() {
    var mediaAccts = this.acUtils.getAccountsByInterface("flockIMediaWebService");
    var acctsMenu = $("flock-media-accounts-dropdown");
    while (acctsMenu.hasChildNodes()) {
      acctsMenu.removeChild(acctsMenu.lastChild);
    }
    if(!mediaAccts.hasMoreElements()) {
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("disabled", "true");
      menuitem.setAttribute("label", flockGetString("photo/mediabar",
                                                    "flock.media.noaccounts"));
      acctsMenu.appendChild(menuitem);
    }
    while (mediaAccts.hasMoreElements()) {
      var acct = mediaAccts.getNext()
                           .QueryInterface(Components.interfaces.flockIWebServiceAccount);
      var c_acct = this._coop.get(acct.urn);
      var svc = Components.classes[c_acct.serviceId]
                          .getService(Components.interfaces.flockIWebService);
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("class", "menuitem-iconic");
      menuitem.setAttribute("image", c_acct.favicon);
      menuitem.setAttribute("label", c_acct.name);
      menuitem.setAttribute("username", c_acct.name);
      menuitem.setAttribute("querystring", "user:" + c_acct.accountId + "|username:" + c_acct.name);
      menuitem.setAttribute("oncommand", "gPhotoDrawer.mFilter = ''; gPhotoDrawer.loadQuery('" + svc.shortName +
                            "', this.getAttribute('querystring'), this.getAttribute('username')); gPhotoDrawer.emptySearchbox(); ");
      if(svc.shortName == "piczo") {
        // Disable it until they've logged in
        if(!c_acct.isAuthenticated) {
          menuitem.setAttribute("disabled", true);
          var notLoggedInString = flockGetString("photo/mediabar",
                                                 "flock.media.favs.notLoggedIn");
          menuitem.setAttribute("label", c_acct.name
                                         + " "
                                         + notLoggedInString);
        }
      }
      acctsMenu.appendChild(menuitem);
    }
  },

  onSettingsMenu: function pdOnSettingsMenu() {
    if ($("mediabar").getAttribute("showDraggers") == "true") {
      $("toggle-draggers").setAttribute("label",
                                        flockGetString("photo/mediabar",
                                                       "flock.media.draggers.hide"));
    } else {
      $("toggle-draggers").setAttribute("label",
                                        flockGetString("photo/mediabar",
                                                       "flock.media.draggers.show"));
    }

    if ($("mediabar").getAttribute("advControls") == "on") {
      $("toggle-advanced-controls").setAttribute("label", flockGetString("photo/mediabar", "flock.media.controls.hide"));
    } else {
      $("toggle-advanced-controls").setAttribute("label", flockGetString("photo/mediabar", "flock.media.controls.show"));
    }

    if ($("mediabar").getAttribute("location") == "top") {
      $("toggle-location").setAttribute("label", flockGetString("photo/mediabar", "flock.media.controls.bottom"));
    } else {
      $("toggle-location").setAttribute("label", flockGetString("photo/mediabar", "flock.media.controls.top"));
    }
  },

  toggleDraggers: function pdToggleDraggers() {
    if ($("mediabar").getAttribute("showDraggers") == "true") {
      $("mediabar").setAttribute("showDraggers", "false");
    } else {
      $("mediabar").setAttribute("showDraggers", "true");
    }
  },

  toggleAdvControls: function pdToggleAdvControls() {
    if ($("mediabar").getAttribute("advControls") == "on") {
      $("mediabar").setAttribute("advControls", "off");
    } else {
      $("mediabar").setAttribute("advControls", "on");
    }
  },

  toggleFavorite: function pdToggleFavorite() {
    var isFave = true;
    var id = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" + this.mQuery.stringVal;
    var mediaQuery = this._coop.get(id);
    if (mediaQuery) {
      if (!this.removeFavorite(mediaQuery)) {
        return;
      }
      top.updateMediaButton();
      this.updateWatch();
      isFave = false;
      this.mediaObserverService.notifyObservers(null, "refresh-myworld-topmedia", id);
    } else {
      this.addFavorite(id);
    }
    this.metrics.report("MediaBar-ToggleFavorite", [{service: this.mApi,
                                                     favorite: isFave}]);
  },

  toggleSize: function pdToggleSize() {
    var newSize;
    if ($("mediabar").getAttribute("photoSizeLarge") == "true") {
      newSize = "Small";
      this.setPhotoSize("small");
    } else {
      newSize = "Large";
      this.setPhotoSize("large");
    }
    this.metrics.report("MediaBar-Options-SwitchView", newSize);
    this._resizePhotos();
  },

  toggleMode: function pdToggleMode() {
    if ($("mediabar").getAttribute("mode") == "consume") {
      this.interactMode();
      this.mGroupbarBox.removeAttribute("hidden");
    } else {
      this.consumeMode();
      this.mGroupbarBox.setAttribute("hidden", "true");
    }
  },

  setPhotoSize: function pdSetPhotoSize(aSize) {
    if (aSize == "large") {
      $("photo-drawer-size").setAttribute("checked", "true");
      this.mSize = "medium";
      top.flock_topbarSize(height_large);
      $("mediabar").setAttribute("photoSizeLarge", true);
    } else {
      $("photo-drawer-size").setAttribute("checked", "false");
      this.mSize = "thumb";
      if (this.getMode() == "consume") {
        top.flock_topbarSize(height_small);
      } else {
        top.flock_topbarSize(height_small_meta);
      }
      $("mediabar").removeAttribute("photoSizeLarge");
    }
  },

  consumeMode: function pdConsumeMode() {
    $("mediabar").setAttribute("mode", "consume");
    if ($("mediabar").getAttribute("photoSizeLarge") != "true") {
      top.flock_topbarSize(height_small);
    }
  },

  interactMode: function pdInteractMode() {
    $("mediabar").setAttribute("mode", "interact");
    if ($("mediabar").getAttribute("photoSizeLarge") != "true") {
      top.flock_topbarSize(height_small_meta);
    }
  },

  getMode: function pdGetMode() {
    return $("mediabar").getAttribute("mode");
  },

  _resizePhotos: function pd_resizePhotos() {

    for (var i = 0; i < this.mGroupbarBox.childNodes.length; i++) {
      this.mGroupbarBox.childNodes[i].setAttribute("width", 0);
      this.mGroupbarBox.childNodes[i].setAttribute("photowidth", 0);
      this.mGroupbarBox.childNodes[i].setAttribute("hidden", true);
    }

    var photos = this.mSlideybox.getElementsByTagName("photo");
    for (var i = 0; i < photos.length; i++) {
      photos[i].sizeProcessed = null;
      photos[i].size = this.mSize;
    }
  },

  processPhotoSize: function pdProcessPhotoSize(aPhoto) {
    // if the photo loads before the groupbar exists, then
    // bail and don't do this twice
    if (aPhoto.sizeProcessed || aPhoto.boxObject.width == 0) {
      return;
    }

    var padding = (aPhoto.mSize == "medium") ? 4 : 4;
    // if the photo is part of a groupbar, and it's the first photo,
    // add its size, plus left AND right edges

    if (aPhoto.getAttribute("groupbar")) {
      if (!$(aPhoto.getAttribute("groupbar"))) {
        return;
      }
      $(aPhoto.getAttribute("groupbar")).changeSize(aPhoto.boxObject.width + padding);
    }

    // if it doesn't have the same groupbar setting as the last one,
    // Close off the previous groupbar size
    if (aPhoto.previousSibling
        && aPhoto.previousSibling.getAttribute("groupbar")
        && aPhoto.previousSibling.getAttribute("groupbar") != aPhoto.getAttribute("groupbar"))
    {
        $(aPhoto.previousSibling.getAttribute("groupbar")).changeSize(-2);
        $(aPhoto.previousSibling.getAttribute("groupbar")).setupBar();
    }
    aPhoto.sizeProcessed = true;
  },

  _loadQuery: function pd_loadQuery(aService, aQuery, aName) {
    this.currentCoopStream = null;
    this.mApi = aService;
    if (aService == "preview") {
      $("mediabar").setAttribute("newMediaPreview", "true");
    } else {
      $("mediabar").removeAttribute("newMediaPreview");
    }
    this.mQuery = aQuery;
    if (this.mQuery.search && this.mQuery.user) {
      this.mFilter = this.mQuery.search;
    }
    this.mName = aName;
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(aService);

    if (svc instanceof Components.interfaces.flockISocialWebService) {
      $("mediabar").setAttribute("social", "true");
    } else {
      $("mediabar").removeAttribute("social");
    }
    this.disableWatch(true);
    this.loadFirst(true);

    // If it's pollable, clear the new flag
    // How to get the higest seq out of the query?
    // I guess (ugly) we register a callback to appendMediaItem
    var uri = FAV_CHANNELS_ROOT + ":" + aService + ":" + aQuery.canonicalVal;
    var item = this._coop.get(uri);
    if (item && item.isPollable) {
      // JMC - Bug #6733
      // If the unseenItems flag is getting cleared,
      // the template will remove the item from one rule,
      // but won't rerun against the other run
      // so we have to manually call rebuild
      var rebuilding = false;
      if (item.hasUnseenItems) {
        rebuilding = true;
      }
      item.hasUnseenItems = false;
      item.markAllSeen();
      if (rebuilding) {
         $("flock-favorites-channels-dropdown").builder.rebuild();
      }
      // JMC - If this has a performance impact, we'll have to defer it for later
    }
    this.currentCoopStream = item;
  },

  clearAllNew: function pdClearAllNew() {
    // JMC - Should this only clear all new that's reflected in the current view?
    // For now, it clears all new mediaqueries no matter what
    var mediaQueries = this._coop.MediaQuery.find({hasUnseenItems: true});
    for (x in mediaQueries) {
      mediaQueries[x].hasUnseenItems = false;
      mediaQueries[x].markAllSeen();
    }

    // DP - Bug #8082
    // if we are on the preview stream and user clears new, load
    // the most recent non-preview stream
    if (this.mApi == "preview") {
      initialMedia = null;
      var inst = this;
      setTimeout(function() { inst.loadInitialMedia(); }, CLEAR_NEW_LOAD_DELAY);
    }
    $("flock-favorites-channels-dropdown").builder.rebuild();
  },

  popMessageBar: function popMessageBarFunction(aQueryString) {
    // Find queryString in a fav'd MediaQuery
    var favItem = this._coop.MediaQuery.find({query: aQueryString, isPollable: true});
    if (favItem.length == 0) {
    var msg = flockGetString("favorites/favorites",
                             "flock.favs.media.discovery");
    var nBox = top.gBrowser.getNotificationBox();
    nBox.appendUniqueNotification(msg,
                                  "media-discovery-info",
                                  "chrome://browser/skin/flock/favorites/media-icon.png",
                                  nBox.FLOCK_PRIORITY_LOW,
                                  null,
                                  null);
    }
  },

  loadQuery: function pdLoadQuery(aService, aQueryString, aName, bIsHistoryNav) {    
    this.metrics.report("MediaBar-LoadMediaStream", "" + aService);
    this.hideActionMessage();
    this.bHistoryNav = (bIsHistoryNav == true);
    this.mFilter = "";
    var aQuery = new queryHelper(aQueryString);
    this.isPersonQuery = aQuery.hasKey("user") || aQuery.hasKey("favorites");

    if ($("mediabar-slideybox").leftmostChild) {
      this.mostRecentSeq = $("mediabar-slideybox").leftmostChild.mPhoto.id;
    } else {
      this.mostRecentSeq = null;
    }

    // we are doing a setTimeout because of a focus issue that happens
    // see flockBug 5827 --- ja
    var inst = this;
    setTimeout(function() { inst._loadQuery(aService, aQuery, aName); } ,0);
  },

  loadFromId: function pdLoadFromId(aId, bIsHistoryNav) {
    this.mFilter = "";
    this.emptySearchbox();
    var item = this._coop.get(aId);
    this.currentItem = item;
    // load it
    // JMC: Special case for special streams, don't use item.name for the name
    var queryName = item.name;
    var itemQuery = new queryHelper(item.query);
    this.loadQuery(item.service, item.query, queryName, bIsHistoryNav);
    
    if (bIsHistoryNav) {
      // update the back/forward buttons
      var backItem = ($(aId).nextSibling && $(aId).nextSibling.tagName == "menuitem");
      var forwardItem = ($(aId).previousSibling && $(aId).previousSibling.tagName == "menuitem") ;
      this._disableMediabarBackButton(!backItem);
      this._disableMediabarForwardButton(!forwardItem);
    }
    
    if (itemQuery.search) {
      this.searchPerson(item.service, itemQuery.search);
    }
    
  },

  /**
   * Navigation via the back and forward arrows.
   * @param aBackwards  boolean: true == go backwards (ex: back arrow clicked)
   */
  navigate: function pdNavigate(aBackwards) {
    //var currentItem = this._coop.get(RECENT_ROOT + ":" + this.mApi + ":" +
    //                                 this.mQuery.stringVal);
    var kids = this._coop.get(RECENT_ROOT).children;
    var recentEnum;
    if (aBackwards) {
      // Yes, this is unfortunately both unintuitive and correct.
      recentEnum = kids.enumerate();
    } else {
      recentEnum = kids.enumerateBackwards();
    }
    var item;
    var itemId;
    var gotToCurrent = false;
    while (recentEnum.hasMoreElements()) {
      item = recentEnum.getNext();
      itemId = item.id();
      if (gotToCurrent && (item.service != "preview" || $("mediabar").getAttribute("hasUnseenMedia"))) {
          this.loadFromId(itemId, true);
          return;
      }
      if (this.currentItem && itemId == this.currentItem.id()) gotToCurrent = true;
      if (!this.currentItem) gotToCurrent = true;
    }
    
    // JMC: This is done inside loadFromId, so it's not required?
    /*
     // Now see if we need to enable or disable the arrows.
    if (recentEnum.hasMoreElements()) {
      // We're no longer at the end.
      if (aBackwards) {
        this._disableMediabarForwardButton(false);
      } else {
        this._disableMediabarBackButton(false);
      }
    } else {
      // We're at one end of the history.
      if (aBackwards) {
        this._disableMediabarBackButton(true);
        this._disableMediabarForwardButton(false);
      } else {
        this._disableMediabarForwardButton(true);
        this._disableMediabarBackButton(false);
      }
    }
    */
  },

  /**
   * Helper for _disableMediabarBackButton and _disableMediabarForwardButton.
   * @param aElement  button to act upon
   * @param aState    true to disable, false to enable
   */
  _disableMediabarNavButton:
  function pd_disableMediabarNavButton (aButton, aState)
  {
    if (aState) {
      aButton.setAttribute("disabled", aState);
    } else {
      if (aButton.hasAttribute("disabled")) {
        aButton.removeAttribute("disabled");
      }
    }
  },

  /**
   * Disables or enables media bar back arrow button
   * @param aState  true to disable, false to enable
   */
  _disableMediabarBackButton:
  function pd_disableMediabarBackButton (aState)
  {
    this._disableMediabarNavButton(this.mediabarBackButton, aState);
  },

  /**
   * Disables or enables media bar forward arrow button
   * @param aState  true to disable, false to enable
   */
  _disableMediabarForwardButton:
  function pd_disableMediabarForwardButton (aState)
  {
    this._disableMediabarNavButton(this.mediabarForwardButton, aState);
  },

  addRecent: function pdAddRecent() {
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(this.mApi);
    // XXX TODO: This is not very robust, and it doesn't allow favorite channels
    // to be recent channels without duplicating the mediaquery
    // Also, the split, below, looks fragile  - similar queries may clobber?

    var detailedQuery = new queryHelper(this.mQuery.stringVal);
   /*
    if (this.mostRecentSeq) {
      detailedQuery.leftmost = this.mostRecentSeq;
    }
    */
    
    // Don't double up at the top.
    if ($('mediabar-recent-channels-popup').childNodes[1] &&
    this._coop.get($('mediabar-recent-channels-popup').childNodes[1].id).query == detailedQuery.stringVal) {
      return;
    }
    
    var item = new this._coop.MediaQuery({
      name: this.mSimpleLabel,
      query: detailedQuery.stringVal,
      service: this.mApi,
      favicon: svc.icon
    });
    if (this.recentRoot.children.enumerate().hasMoreElements()) {
      // We have history.  Enable "back".
      this._disableMediabarBackButton(false);
    }
    this.recentRoot.children.insertAt(item, 1);
    this.cleanRecentList();
    this.currentItem = item;
  },

  addFavorite: function pdAddFavorite(id) {
    var photoAPI = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager);
    var svc = photoAPI.getAPIFromShortname(this.mApi);

    var item = new this._coop.MediaQuery(id, {
      name: this.mSimpleLabel,
      query: this.mQuery.canonicalVal,
      service: this.mApi,
      favicon: svc.getIconForQuery(this.mQuery.canonicalVal),
      isPollable: true,
      hasUnseenItems: false //,
      // latestSeq: this.latestSeq
    });

    //XXX TODO: Should this be addOnce?
    this.favChannelsRoot.children.add(item);
    // Call refresh manually to force a no-newness refresh
    this.updateWatch();
    var lister = {
      onResult: function (a, b) {},
      onError: function (a) {}
    }
    photoAPI.refresh(id, lister);

    if (!this.pref_service.getPrefType(FLOCK_MEDIA_ADD_FAV_DLG_PREF) ||
        (this.pref_service.getBoolPref(FLOCK_MEDIA_ADD_FAV_DLG_PREF) == true))
    {
      var url = "chrome://browser/content/flock/photo/addFavoriteDlg.xul";
      window.openDialog(url, "_blank", "centerscreen,modal");
    }
  },

  removeFavorite: function pdRemoveFavorite(aMediaQuery) {
    var photoAPIManager = CC["@flock.com/photo-api-manager;1?"]
                          .getService(CI.flockIPhotoAPIManager);
    if(photoAPIManager.destroyPhotos(aMediaQuery.id())) {
      aMediaQuery.destroy();
      return true;
    }
    return false;
  },

  showActionMessage: function pdShowActionMessage(aMessage) {
    $("mediabar-bannermessage").message = aMessage;
    $("mediabar-bannermessage").show();
  },

  hideActionMessage: function pdHideActionMessage() {
    $("mediabar-bannermessage").hide();
  },

  updateWatch: function pdUpdateWatch() {
    var id = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" +
             this.mQuery.canonicalVal;
    // XXX: JMC - I have changed the above to allow favoriting of searches
    // within a user. Does this make sense?
    $("mediabar-watch").setAttribute("checked", this._coop.get(id) != null);
  },

  disableWatch: function pdDisableWatch(aState) {
    $("mediabar-watch").setAttribute("disabled", aState);
  },

  cleanRecentList: function pdCleanRecentList() {
    var count = 0;
    var theEnum = this.recentRoot.children.enumerate();
    while (theEnum.hasMoreElements()) {
      theEnum.getNext(); count++;
    }
    // Only one element max to remove
    if (count > MAX_RECENT) {
      var fat = count - MAX_RECENT;
      var enumBack = this.recentRoot.children.enumerateBackwards();
      for (i = 0; i < fat; i++) {
        var item = enumBack.getNext();
        this.recentRoot.children.remove(item);
        item.destroy();
      }
    }
  },

  emptySearchbox: function pdEmptySearchBox() {
    $("mediabar-search-textbox").value = "";
    $("mediabar-search-textbox").setAttribute("empty", "true");
    // this.onSearchboxBlur();
    $("mediabar-search-textbox").blur();
  },

  onSearchboxBlur: function pdOnSearchboxBlur(aForce) {
    if ($("mediabar-search-textbox").value.length == 0 || aForce) {
 /*     if (this.supportsSearch) {
        $("mediabar-search-textbox").value = "Search " + this.mName;
      } else { */
        var mediaSearchApi = this.pref_service
                                 .getCharPref(FLOCK_CURRENT_MEDIA_SEARCH_PREF);
        var txt;
        if (mediaSearchApi == "filter") {
          /* It'd be way better to pull this string once and keep it. */
          txt = flockGetString("photo/mediabar", "flock.media.search.filter");
        } else if (mediaSearchApi.substring(0, 8) == "mediarss") {
          var rssEngine = mediaSearchApi.split(":")[1];
          txt = flockGetString("photo/mediabar",
                               "flock.media.search.title",
                               [this.mRSSEngines[rssEngine].title]);
        } else {
          var svc = CC["@flock.com/photo-api-manager;1?"]
                    .getService(CI.flockIPhotoAPIManager)
                    .getAPIFromShortname(mediaSearchApi);
          txt = flockGetString("photo/mediabar",
                               "flock.media.search.title",
                               [svc.title]);
        }
        $("mediabar-search-textbox").value = txt;
  //    }
      $("mediabar-search-textbox").setAttribute("empty", true);
    }
  },

  onSearchboxFocus: function pdOnSearchboxFocus() {
    if ($("mediabar-search-textbox").hasAttribute("empty")) {
      $("mediabar-search-textbox").removeAttribute("empty");
      $("mediabar-search-textbox").value = "";
    }
  },

  /* this function will hopefully go away when we work in
     our solution to display the # of new items
     */
  updateFavoritesChannelUI: function pdUpdateFavoritesChannelUI() {
    if (this.mApiMgr.hasNewMedia) {
      $("mediabar").setAttribute("hasUnseenMedia", true);
    } else {
      $("mediabar").removeAttribute("hasUnseenMedia");
      // JMC - don't leave preview new queries in recent history
      var mediaQueries = this._coop.MediaQuery.find({service: "preview"});
      for (x in mediaQueries) {
        mediaQueries[x].destroy();
      }
    }
  },

  lock: function pdLock() {
    if (this.objSynch) {
      // if already locked return false; could not lock
      return false;
    }
    this.objSynch = true;
    return true;
  },

  unlock: function pdUnlock() {
    setTimeout(function() { gPhotoDrawer.objSynch = false; }, 500);
  }
}

PhotoDrawer.prototype.__defineGetter__("mLabel",  function () {
  var label = "";
  var suffix = "";
  if (this.mApi == "facebook") {
    suffix = ".facebook";
  }
  
  if (!this.mQuery) {
    return;
  } 
  // ALBUM, USERNAME, OR SEARCHUNIQUE start the label...
  if (this.mQuery.album) {
    label = flockGetString("photo/mediabar", "flock.media.viewing.album" + suffix,
                           [this.mQuery.albumlabel, this.mQuery.username]);
  } else if (this.mQuery.favorites) {
    label = flockGetString("photo/mediabar", "flock.media.viewing.favorites" + suffix,
                             [this.mQuery.favorites]);
  } else if (this.mQuery.username) {
    label = flockGetString("photo/mediabar", "flock.media.viewing.user" + suffix,
                             [this.mQuery.username]);
  } else if (this.mQuery.searchunique) {
    label += flockGetString("photo/mediabar", "flock.media.viewing.searchunique" + suffix,
                           [this.mQuery.username]);
  }
  
  // ... And a search filter might finish it off
  if (this.mApi == "preview") {
    this.mName = flockGetString("photo/mediabar", "flock.media.search.prev.new");
  }
  if (this.mQuery.special || this.mApi == "preview") {
    label += flockGetString("photo/mediabar", "flock.media.viewing.special" + suffix,
                            [this.mName]);
  }
  if (this.mQuery.pool) {
    label += flockGetString("photo/mediabar", "flock.media.viewing.group" + suffix,
                             [this.mQuery.albumlabel]);
  }
  if (this.mQuery.search) {
    if (label) {
      label += " " + flockGetString("photo/mediabar", "flock.media.viewing.filteredby",
                                                  [this.mQuery.search]);
    } else {
      label = flockGetString("photo/mediabar", "flock.media.viewing.search",
                             [this.mQuery.search]);
    }
  }
  return label;
})

PhotoDrawer.prototype.__defineGetter__("mSimpleLabel",  function () {
  var label = "";
  
  if (!this.mQuery) {
    return;
  } 
  // ALBUM, USERNAME, OR SEARCHUNIQUE start the label...
  if (this.mQuery.album) {
    label = flockGetString("photo/mediabar", "flock.media.simpleviewing.album",
                           [this.mQuery.albumlabel, this.mQuery.username]);
  } else if (this.mQuery.favorites) {
    label = flockGetString("photo/mediabar", "flock.media.simpleviewing.favorites",
                             [this.mQuery.favorites]);
  } else if (this.mQuery.username) {
    label = flockGetString("photo/mediabar", "flock.media.simpleviewing.user",
                             [this.mQuery.username]);
  } else if (this.mQuery.searchunique) {
    label = flockGetString("photo/mediabar", "flock.media.simpleviewing.searchunique",
                           [this.mQuery.username]);
  }
  
  // ... And a search filter might finish it off
  if (this.mApi == "preview") {
    this.mName = flockGetString("photo/mediabar", "flock.media.search.prev.new");
  }
  if (this.mQuery.special || this.mApi == "preview") {
    label = this.mName;
  }
  if (this.mQuery.pool) {
    label += flockGetString("photo/mediabar", "flock.media.simpleviewing.group",
                             [this.mQuery.albumlabel]);
  }
  if (this.mQuery.search) {
    if (label) {
      label += " " + flockGetString("photo/mediabar", "flock.media.simpleviewing.filteredby",
                                                  [this.mQuery.search]);
    } else {
      label = flockGetString("photo/mediabar", "flock.media.simpleviewing.search",
                             [this.mQuery.search]);
    }
  }
  return label;
})

PhotoDrawer.prototype.onMediabarChannelsPopupShowing = function() {
  var id = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" + this.mQuery.stringVal;
  if (this._coop.get(id)) {
    $("favorite-this-stream-menuitem").setAttribute("label",
                                                    flockGetString("photo/mediabar", "flock.media.favs.remove"));
  } else {
    $("favorite-this-stream-menuitem").setAttribute("label",
                                                    flockGetString("photo/mediabar", "flock.media.favs.add"));
  }

  // Disable "Add this stream" menuitem if on Preview New
  if (this.mApi  == "preview") {
    $("favorite-this-stream-menuitem").setAttribute("disabled", "true");
  } else {
    $("favorite-this-stream-menuitem").removeAttribute("disabled");
  }

  // disable the Favorite Channels menu if empty
  var thisenum = this.favChannelsRoot.children.enumerate();
  $("favChannelMenu").setAttribute("disabled", "true");
  $("allNewMedia").setAttribute("disabled", "true");
  $("clearNew").setAttribute("disabled", "true");
  while (thisenum.hasMoreElements()) {
    $("favChannelMenu").removeAttribute("disabled");
    var favitem = thisenum.getNext();
    if (favitem && favitem.hasUnseenItems) {
      $("allNewMedia").removeAttribute("disabled");
      $("clearNew").removeAttribute("disabled");
    }
  }

  if (this.mApiMgr.hasNewMedia) {
    $("favChannelMenu").setAttribute("label", flockGetString("photo/mediabar", "flock.media.favs.nameNew"));
  } else {
    $("favChannelMenu").setAttribute("label", flockGetString("photo/mediabar", "flock.media.favs.name"));
  }
}

PhotoDrawer.prototype.setAlbumName = function(albumId, albumLabel) {
  this.mQuery.album = albumId;
  this.mQuery.albumlabel = albumLabel;
  this.mAlbumLabel = albumLabel;
  this._updateCurrentStreamLabels();
}

PhotoDrawer.prototype.browseAlbum = function pd(eventTarget) {
  if (eventTarget.target.localName != "menuitem") {
    return;
  }

  var albumId = eventTarget.target.getAttribute("value");
  var albumLabel = eventTarget.target.getAttribute("label")

  this.clear();
  this.mFilter = "";
  this.mQuery.search = "";
  this.emptySearchbox();
  this.setAlbumName(albumId, albumLabel);
  this.setupSearch();
  this.loadMore();
}

PhotoDrawer.prototype.setupSearch = function() {
  var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                      .getService(Components.interfaces.flockIPhotoAPIManager)
                      .getAPIFromShortname(this.mApi);
  $("mediabar-statusbox").busy(flockGetString("photo/mediabar",
                                              "flock.media.search.searching")
                               + " "
                               + svc.title);
  this.supportsSearch = svc.supportsSearch(this.mQuery.stringVal);
  if (this.supportsSearch) {
    $('media-search-filter-menuitem').removeAttribute('disabled');
  } else {
    $('media-search-filter-menuitem').setAttribute('disabled', true);
    var mediaSearchApi = this.pref_service
                             .getCharPref(FLOCK_CURRENT_MEDIA_SEARCH_PREF);
    if (mediaSearchApi == 'filter') {
      $('media-search-filter-menuitem').nextSibling.doCommand();
    }
  }
}

function onLoad_photo(aEvt) {
  gPhotoDrawer = new PhotoDrawer($("mediabar"), $("mediabar-slideybox"));
  if (gInitialActionMessage) {
    gPhotoDrawer.showActionMessage(gInitialActionMessage);
    gInitialActionMessage = null;
  }
  $("mediabar-refresh").addEventListener("command", onMediaRefresh, false);
  gPhotoDrawer.move(true);
}

function onUnload_photo(aEvt) {
  gDomFx.uninit();
  if (gPhotoDrawer) {
    gPhotoDrawer.uninit();
  }
}

function onClick_photo(aEvt) {
  $("mediabar-bannermessage").hide();
}

function onMediaRefresh(aEvt) {
  if(gPhotoDrawer.lock()) {
    gPhotoDrawer.refreshView();
    gPhotoDrawer.unlock();
  }
}

var updateTimer;
var unseenItemsObs = {
  rdfChanged: function (ds, type, source, predicate, target, oldtarget) {
    // JMC - Just set a dirty flag and update the UI on a timer
    // Otherwise this will really hurt when changing a lot of streams.
    // XXX TODO: Fix this for multi window bugs
    try {
      if (updateTimer) {
        clearTimeout(updateTimer);
      }
      updateTimer = setTimeout("gPhotoDrawer.updateFavoritesChannelUI();", 1);
    } catch(e) { dump(e); }
  }
}

function PhotoDropProcessor(aList, aHoverChild)  {
  this.mList = aList;
  this.mHoverChild = aHoverChild;
}

PhotoDropProcessor.prototype = {
  mList: null,
  start: function pdpStart() {
    var str = "";
    if (this.mHoverChild) {
      var newThing = Components.classes["@flock.com/photo-upload;1"]
                               .createInstance(Components.interfaces.flockIPhotoUpload);
      newThing.id = this.mHoverChild.getAttribute("id");
      this.mHoverChild = newThing;
    }
    photoUploadSvc.addListener(this);
    this.next();
  },

  next: function pdpNext() {
    if(this.mList.length==0) {
      photoUploadSvc.startImportQueue(this);
      return;
    }
    var url = this.mList.shift();
    if (url.split(":")[0] == "file") {
      photoUploadSvc.addMediaToImportQueue(url, this);
      this.next();
    } else {
      // dnding from a remote source
      var URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"]
                               .getService(Components.interfaces.nsIURIFixup);
      var uri = URIFixup.createFixupURI(url, URIFixup.FIXUP_FLAG_NONE);

      var IOService = Components.classes["@mozilla.org/network/io-service;1"]
                                .getService(Components.interfaces.nsIIOService);
      var channel = IOService.newChannelFromURI(uri);

      var self = this;

      var streamListener = {
        processor: self,
        sinkFile: null,
        sinkBufferStream: null,

        onStartRequest: function onStartRequest(request, context) {
          var extension;
          switch (channel.contentType) {
            case "image/jpeg":
              extension = "jpg";
              break;
            case "image/png":
              extension = "png";
              break;
            case "image/gif":
              extension = "gif";
              break;
            default:
              extension = "jpg";
              break;
          }
          var sinkFileName = url.split("/").pop();

          this.sinkFile = Components.classes["@mozilla.org/file/directory_service;1"]
                                    .getService(Components.interfaces.nsIProperties)
                                    .get("ProfD", Components.interfaces.nsIFile);
          this.sinkFile.append("flock_imagecache");
          if (!this.sinkFile.exists()) {
            this.sinkFile.createUnique(this.sinkFile.DIRECTORY_TYPE, 0700);
          }
          if (sinkFileName) {
            this.sinkFile.append(sinkFileName);
          }

          var sinkFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                                         .createInstance(Components.interfaces.nsIFileOutputStream);
          sinkFileStream.init(this.sinkFile, 0x02 | 0x08 | 0x20, 0664, 0);
          this.sinkBufferStream = Components.classes["@mozilla.org/network/buffered-output-stream;1"]
                                            .createInstance(Components.interfaces.nsIBufferedOutputStream);
          this.sinkBufferStream.init(sinkFileStream, 65536 * 4);
        },

        onStopRequest: function onStopRequest(request, context, statusCode) {
          var ios = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);
          var fileHandler = ios.getProtocolHandler("file")
                               .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
          this.sinkBufferStream.close();
          photoUploadSvc.addMediaToImportQueue(fileHandler.getURLSpecFromFile(this.sinkFile), this.processor);
          this.processor.next();
        },

        onDataAvailable:
        function onDataAvailable(request, context, inputStream, offset, count) {
          this.sinkBufferStream.writeFrom(inputStream, count);
        }
      }

      channel.asyncOpen(streamListener, null);
    }
  },

  onMediaBatchImportStart: function pdpOnMediaBatchImportStart(aNumImport) {
  },

  onMediaBatchImportFinish: function pdpOnMediaBatchImportFinish() {
    photoUploadSvc.removeListener(this);
  },

  onMediaImportStart: function onMediaImportStart(aFilename, aNum) {
  },

  onMediaImportFinish: function onMediaImportFinish(aPhotoUpload) {
    photoUploadSvc.addPhotoUpload((this.mHoverChild?this.mHoverChild:null),aPhotoUpload);
  },

  onMediaImportError: function onMediaImportError(aFile) {
    var stringBundle = flock_photo.util.getStringBundle();
    alert(stringBundle.GetStringFromName("flock.photo.upload.error.invalid").replace("%1",aFile));
    this.next();
  },

  onUploadAdd: function pdpOnUploadAdd() {
  },

  onUploadRemove: function pdpOnUploadRemove() {
  },

  onError: function pdpOnError(aError) {
    alert(aError.errorString);
    this.next();
  },

  cancel: function pdpCancel() {
    // not impl
  }
}

PhotoDrawer.prototype.setupAlbumMenu = function() {
  this.logger.info(".setupAlbumMenu()");
  var inst = this;
  var albumsMenu = {
    popup: $("current-stream-popup"),
    numAlbums: 0,

    init: function() {
      $("current-stream-menu").setAttribute("hidden", "true");
      $("current-stream-menuitem").removeAttribute("hidden");

      var numItems = this.popup.childNodes.length;

      for (var i = 0; i < numItems; i++) {
        if (!this.popup.lastChild.hasAttribute("dontDelete")) {
          this.popup.removeChild(this.popup.lastChild);
        }
      }
    },

    refresh: function(albumEnumerator) {
      this.init();
      var albumId;
      var albumTitle;
      this.numAlbums = 0;
      while(albumEnumerator.hasMoreElements()) {
        $("current-stream-menuitem").setAttribute("hidden", "true");
        $("current-stream-menu").removeAttribute("hidden");

        var album = albumEnumerator.getNext();
        album.QueryInterface(Components.interfaces.flockIPhotoAlbum);
        var menuitem = document.createElement("menuitem");
        this.popup.appendChild(menuitem);
        if (this.album == album.id) {
          inst.setAlbumName(album.id, album.title);
        }
        menuitem.setAttribute("value", album.id);
        menuitem.setAttribute("label", album.title);
        this.numAlbums++;
      }
    }
  }

  var albumsGetterListener = {
    onStart: function setupAlbumMenu_agl_onStart(aSubject, aTopic) {
    },
    onSuccess: function setupAlbumMenu_agl_onSuccess(aSubject, aTopic) {
      aSubject.QueryInterface(CI.nsISimpleEnumerator);
      albumsMenu.refresh(aSubject);
    },
    onError: function setupAlbumMenu_agl_onError(aSubject, aTopic, aError) {
      var msg = aError ? aError.errorString 
                       : ("getAlbums() is not implemented by " + inst.mApi);
      inst.logger.error(msg);
    }
  };

  this._updateCurrentStreamLabels();
  albumsMenu.init();

  var userId = this.mQuery.getKey("user");

  if (userId && userId.length) {
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(this.mApi);
    svc.getAlbums(albumsGetterListener, userId);
  }
}

PhotoDrawer.prototype.onMediaTooltipShowing = function() {
  // if DnD in progress don't show the rich photo tooltip
  // see https://bugzilla.flock.com/show_bug.cgi?id=8559
  if (this.mDnDInProgress) {
    return false;
  }
}

PhotoDrawer.prototype._updateCurrentStreamLabels =
function PhotoDrawer_updateCurrentStreamLabels() {
  this.logger.debug("._updateCurrentStreamLabels()");
  $("current-stream-label").setAttribute("value", this.mLabel);
  $("current-stream-menu").setAttribute("label", this.mLabel);
  $("current-stream-menuitem").setAttribute("label", this.mLabel);
   $("current-stream-icon").setAttribute("src", this.currentIcon());
};

PhotoDrawer.prototype.currentIcon =
function PhotoDrawer_currentIcon() {
  if (this.mApi == "preview") {
    return "";
  }
  var svc = CC["@flock.com/photo-api-manager;1?"]
            .getService(CI.flockIPhotoAPIManager)
            .getAPIFromShortname(this.mApi);
  if (svc instanceof CI.flockIMediaWebService) {
    return svc.getIconForQuery(this.mQuery.canonicalVal);
  }
  return "";
}

window.addEventListener("load", onLoad_photo, false);
window.addEventListener("unload", onUnload_photo, false);
window.addEventListener("click", onClick_photo, false);
