// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const CC = Components.classes;
const CI = Components.interfaces;
const CR = Components.results;
const CU = Components.utils;

CU.import("resource:///modules/FlockXPCOMUtils.jsm");
FlockXPCOMUtils.debug = false;
CU.import("resource:///modules/FlockSvcUtils.jsm");
CU.import("resource:///modules/FlockXMLUtils.jsm");
CU.import("resource://gre/modules/JSON.jsm");

const MODULE_NAME = "Pownce";
const CLASS_NAME = "Flock Pownce Service";
const CLASS_SHORT_NAME = "pownce";
const CLASS_TITLE = "Pownce";
const CLASS_ID = Components.ID("{7C8077D0-B7E0-4F76-9E69-58BBACEBB3C2}");
const CONTRACT_ID = "@flock.com/webservice/pownce;1";
const FLOCK_ERROR_CONTRACTID = "@flock.com/error;1";
// From nsIXMLHttpRequest.idl
const XMLHTTPREQUEST_READYSTATE_COMPLETED = 4;
const HTTP_CODE_OK = 200;
const HTTP_CODE_FOUND = 302;
const FLOCK_RICH_DND_CONTRACTID = "@flock.com/rich-dnd-service;1";

const SERVICE_ENABLED_PREF = "flock.service.pownce.enabled";

const POWNCE_URL = "http://pownce.com/";
const FAVICON = "chrome://flock/content/services/pownce/favicon.png";

// The delay between two refreshes when the sidebar is closed (in seconds)
const REFRESH_INTERVAL = 1800; // seconds == 30 minutes
// The delay between two refreshes when the sidebar is open (in seconds)
const SHORT_INTERVAL = 300; // seconds == 5 minutes

const POWNCE_API_URL = "http://api.pownce.com/2.1/";

const POWNCE_FRIENDS_PAGE_SIZE = 20;
const POWNCE_MESSAGES_PAGE_SIZE = 20;

const POWNCE_MAX_STATUS_LENGTH = 4000;

const POWNCE_PROPERTIES = "chrome://flock/locale/services/pownce.properties";

const POWNCE_FLOCK_APP_KEY = "0ngf12r8x98659z180m1o7frcq31d96h";
const POWNCE_FLOCK_APP_SECRET = "pu62n713j20t5m1e828vqx70o32o351a";

var gApi = null;
var gSvc = null;


/*************************************************************************
 * Component: PownceAPI
 *************************************************************************/
function PownceAPI() {
  FlockSvcUtils.getLogger(this);
  this._logger.init("PownceAPI");

  FlockSvcUtils.getACUtils(this);
  FlockSvcUtils.getCoop(this);

  this._logger.debug("constructor");
}


/*************************************************************************
 * PownceAPI: XPCOM Component Creation
 *************************************************************************/
// Use genericComponent() for the goodness it provides (QI, nsIClassInfo, etc),
// but do NOT add this component to the list of constructors passed to
// FlockXPCOMUtils.genericModule().
PownceAPI.prototype = new FlockXPCOMUtils.genericComponent(
  CLASS_NAME + " API",
  "",
  "",
  PownceAPI,
  0,
  []
  );


/*************************************************************************
 * PownceAPI: flockIWebServiceAPI Implementation
 *************************************************************************/

/**
 * void call(in AString aMethod,
 *           in nsISupports aParams,
 *           in nsISupports aPostVars, // not part of flockIWebServiceAPI
 *           in flockIListener aListener);
 * @see flockIWebServiceAPI#call
 */
PownceAPI.prototype.call =
function PownceAPI_call(aMethod, aParams, aPostData, aListener) {
  // If post data was passed in, assume this request is a POST.
  var isPost = (aPostData) ? true : false;

  var responseFormat = ".json";
  var url = POWNCE_API_URL + aMethod + responseFormat;
  var error = {};

  // All authenticated Pownce API requests must include the app_key
  var params = (aParams) ? aParams : {};
  params.app_key = POWNCE_FLOCK_APP_KEY;

  var idx = 0;
  for (var p in params) {
    // Only use "?" for the first param.  Use "&" after.
    url += (idx == 0) ? "?" : "&";
    url += p + "=" + params[p];
    idx++;
  }

  this._logger.debug(".call() url: " + url);

  var req = CC["@mozilla.org/xmlextras/xmlhttprequest;1"]
            .createInstance(CI.nsIXMLHttpRequest)
            .QueryInterface(CI.nsIJSXMLHttpRequest);

  // Don't pop nsIAuthPrompts if auth fails
  req.backgroundRequest = true;

  req.open((isPost) ? "POST" : "GET", url, true);

  if (this.isAuthenticated) {
    var acct = gSvc.getAuthenticatedAccount();
    var passwordUrn = "urn:pownce:service:" + acct.getParam("accountId");
    var creds = this._acUtils.getPassword(passwordUrn);

    // XXX: This is a gross hack to use the window.btoa() Base64 method.
    //      Remove when we're using Gecko 1.9, where btoa() becomes a global.
    var win = CC["@mozilla.org/appshell/window-mediator;1"]
              .getService(CI.nsIWindowMediator)
              .getMostRecentWindow("navigator:browser");

    // Pownce wants the authn creds as base64(username:password)
    var credString = win.btoa(creds.user + ":" + creds.password);
    req.setRequestHeader("Authorization", "basic " + credString);
    this._logger.debug(".call() This is authenticated as user "
                       + creds.user);
  } else {
    this._logger.debug(".call() This is unauthenticated");
  }

  var inst = this;
  req.onreadystatechange = function api_call_orsc(aEvent) {
    inst._logger.debug(".call() ReadyState: " + req.readyState);

    if (req.readyState == XMLHTTPREQUEST_READYSTATE_COMPLETED) {
      try {
        inst._logger.debug(".call() Status: " + req.status);

        if (req.status/100 == 2) {
          inst._logger.debug(".call() response:\n" + req.responseText);
          var s = new CU.Sandbox("about:blank");
          var result = CU.evalInSandbox("(" + req.responseText + ")", s);

          if (result) {
            // Pownce can return a 200 with an error described in the JSON.
            if (result.error) {
              error = inst.getError(result.error.status_code);
              error.serviceErrorCode = result.error.status_code;
              error.serviceErrorString = result.error.message;
              aListener.onError(null, arguments.callee.name, error);
            } else {
              aListener.onSuccess(result, arguments.callee.name);
            }
          } else {
            inst._logger.debug(".call() error: "
                               + "empty or poorly formatted result");
            error = inst.getError(null);
            aListener.onError(null, arguments.callee.name, error);
          }
        } else {
          // HTTP errors
          inst._logger.debug(".call() HTTP Error: " + req.status);
          error = inst.getHttpError(req.status);
          error.serviceErrorString = req.responseText;
          aListener.onError(null, arguments.callee.name, error);
        }

      } catch (ex) {
        // XMLHTTPERROR (connection lost)
        inst._logger.debug(".call() exception: " + ex);
        error = inst.getHttpError(9001);
        error.serviceErrorString = "exception (connection lost?): " + ex;
        aListener.onError(null, arguments.callee.name, error);
      }
    }
  };

  if (isPost) {
    req.setRequestHeader("Content-Type",
                         "application/x-www-form-urlencoded; charset=UTF-8");
    var postBody = "";
    for (var key in aPostData) {
      if (postBody.length) {
        postBody += "&";
      }
      postBody += key + "=" + encodeURIComponent(aPostData[key]);
    }
  }

  if (isPost && postBody && postBody.length) {
    this._logger.debug(".call() postBody: " + postBody);
    req.send(postBody);
  } else {
    req.send(null);
  }
};

/**
 * flockIError getError(in AString aErrorCode);
 * @see flockIWebServiceAPI#getError
 */
PownceAPI.prototype.getError =
function PownceAPI_getError(aErrorCode) {
  this._logger.debug(".getError('" + aErrorCode + "')");
  var error = CC[FLOCK_ERROR_CONTRACTID].createInstance(CI.flockIError);

  switch (aErrorCode) {
    default:
      error.errorCode = CI.flockIError.PHOTOSERVICE_UNKNOWN_ERROR;
  }
  return error;
};

/**
 * flockIError getHttpError(in AString aHttpErrorCode);
 * @see flockIWebServiceAPI#getHttpError
 */
PownceAPI.prototype.getHttpError =
function PownceAPI_getHttpError(aHttpErrorCode) {
  this._logger.debug(".getHttpError('" + aHttpErrorCode + "')");
  var error = CC[FLOCK_ERROR_CONTRACTID].createInstance(CI.flockIError);
  error.errorCode = aHttpErrorCode;
  error.serviceErrorCode = aHttpErrorCode;
  return error;
};


/*************************************************************************
 * PownceAPI: flockIAuthenticatedAPI Implementation
 *************************************************************************/

// readonly attribute boolean isAuthenticated;
PownceAPI.prototype.isAuthenticated = false;

/**
 * void authenticate(in nsIPassword aCredentials,
 *                   in flockIListener aListener);
 * @see flockIAuthenticatedAPI#authenticate
 */
PownceAPI.prototype.authenticate =
function PownceAPI_authenticate(aCredentials, aListener) {
  var logMessage = ".authenticate(";
  logMessage += (aCredentials) ? "'" + aCredentials.user + "', " : "";
  logMessage += "...)";
  this._logger.debug(logMessage);
  this.isAuthenticated = true;
  if (aListener) {
    aListener.onSuccess(this, arguments.callee.name);
  }
};

/**
 * void deauthenticate();
 * @see flockIAuthenticatedAPI#deauthenticate
 */
PownceAPI.prototype.deauthenticate =
function PownceAPI_deauthenticate() {
  this._logger.debug(".deauthenticate()");
  this.isAuthenticated = false;
};


/*************************************************************************
 * PownceAPI: Private data and functions
 *************************************************************************/
/**
 * Ensure the API call is authenticated, or don't perform it at all.
 * void authnCall(in AString aMethod,
 *                in nsISupports aParams,
 *                in nsISupports aPostVars,
 *                in flockIListener aListener);
 * @see flockIWebServiceAPI#call
 */
PownceAPI.prototype.authnCall =
function PownceAPI_authnCall(aMethod, aParams, aPostVars, aListener) {

  var inst = this;
  var authnListener = {
    onSuccess: function authnListener_onSuccess(aSubject, aTopic) {
      inst.call(aMethod, aParams, aPostVars, aListener);
    },
    onError: function authnListener_onError(aSubject, aTopic, aError) {
      // log error and just bail
      inst._logger.error(arguments.callee.name)
    }
  };

  if (!this.isAuthenticated) {
    this.authenticate(null, authnListener);
  } else {
    authnListener.onSuccess(null, null);
  }
};

/**
 * Get profile info about a user.
 * @param  aUsername  Pownce username of the user to query.
 * @param  aApiListener
 */
PownceAPI.prototype.userShow =
function PownceAPI_userShow(aUsername, aApiListener) {
  this._logger.debug(".userShow(" + aUsername + ")");

  var url = "users/" + aUsername;
  gApi.authnCall(url, null, null, aApiListener);
};


/**
 * Gets the latest public note posted by a user.
 * @param  aUsername  Pownce username of the user to query.
 * @param  aApiListener
 */
PownceAPI.prototype.getLastPublicNoteByUser =
function PownceAPI_getLastPublicNoteByUser(aUsername, aApiListener) {
  this._logger.debug(".getLastPublicNoteByUser('" + aUsername + "', ...)");

  var url = "note_lists/" + aUsername
  var params = {
    type: "messages",
    filter: "sent",
    limit: "1"
  };
  gApi.authnCall(url, params, null, aApiListener);
};

/**
 * Get a user's friends' updates.
 * @param  aUsername  Pownce username of the user whose friends to view.
 * @param  aApiListener
 */
PownceAPI.prototype.getFriendsStatus =
function PownceAPI_getFriendsStatus(aUsername, aApiListener) {
  this._logger.debug(".getFriendsStatus(" + aUsername + ", ...)");

  var peopleHash = {};
  var inst = this;

  var params = {};
  params.page = 0;
  params.with_notes = 1; // include latest message with each friend's details

  var url = "users/" + aUsername + "/friends";
  var friendsLoopApiListener = {
    onSuccess: function L_onSuccess(aResult) {
      if (!(aResult && aResult.friends)) {
        aApiListener.onError(null, "friendsLoopApiListener", null);
        return;
      }
      var friendsResult = aResult.friends;

      // If we get users back, lets check the next page.
      if (friendsResult.has_next_page) {
        params.page++;
        gApi.authnCall(url, params, null, friendsLoopApiListener);
      }

      for each (var userObj in friendsResult.users) {
        var id = userObj.id;
        peopleHash[id] = userObj;
        inst._logger.debug("Found Pownce person: '"
                           + peopleHash[id].id + "' '"
                           + peopleHash[id].username + "'");
      }
      aApiListener.onSuccess(peopleHash);
    },
    onError: function L_onError(aError) {
      inst._logger.debug("friendsLoopApiListener() onError");
      aApiListener.onError(aError);
    }
  };

  gApi.authnCall(url, params, null, friendsLoopApiListener);
};

/**
 * Set the user's status
 * @param  aStatusMessage  a string containing the message to set
 * @param  aApiListener
 */
PownceAPI.prototype.setStatus =
function PownceAPI_setStatus(aStatusMessage, aApiListener) {
  this._logger.debug(".setStatus('" + aStatusMessage + "', ...)");

// From Pownce API 2.0 documentation:
//
// Post a note that contains only a message.
//   url: http://api.pownce.com/2.0/send/message.{format}
//   request methods: POST
//   authentication: HTTP basic, OAuth
//   required url parameters:
//     {format}  The expected response format. Options are xml, or json.
//     {app_key} The application key assigned during registration.
//   required POST parameters:
//     {note_to} The recipient(s) of the note. Options are public, all,
//               friend_x, or set_x. Available options for the authenticated
//               user can be found from the Send To List endpoint.
//     {note_body} The main text body of the note.
//   notes: Returns the note that was posted in the requested response format.

  var url = "send/message";

  var postData = {
    note_to: "public",
    note_body: aStatusMessage
  };

  gApi.authnCall(url, null, postData, aApiListener);
};


/*************************************************************************
 * Component: PownceService
 *************************************************************************/
function PownceService() {
  FlockSvcUtils.getLogger(this);
  this._logger.init("PownceService");

  // Determine whether this service has been disabled, and unregister if so.
  var prefService = CC["@mozilla.org/preferences-service;1"]
                    .getService(CI.nsIPrefBranch);
  if (prefService.getPrefType(SERVICE_ENABLED_PREF) &&
      !prefService.getBoolPref(SERVICE_ENABLED_PREF))
  {
    this._logger.info(SERVICE_ENABLED_PREF + " is FALSE! Not initializing.");
    this.deleteCategories();
    return;
  }

  var profiler = CC["@flock.com/profiler;1"].getService(CI.flockIProfiler);
  var evtID = profiler.profileEventStart("pownce-init");

  this._obs = CC["@mozilla.org/observer-service;1"]
              .getService(CI.nsIObserverService);
  this._obs.addObserver(this, "xpcom-shutdown", false);

  this._accountClassCtor = PownceAccount;

  FlockSvcUtils.getCoop(this);

  FlockSvcUtils.getACUtils(this);
  FlockSvcUtils.getWD(this);

  this.ppUtils = CC["@flock.com/people-utils;1"]
                 .getService(CI.flockIPeopleUtils);

  // Initialize API
  gApi = new PownceAPI();

  // Update auth states
  try {
    if (this._WebDetective.detectCookies(CLASS_SHORT_NAME, "loggedout", null))
    {
      this._acUtils.markAllAccountsAsLoggedOut(CONTRACT_ID);
      gApi.deauthenticate();
    }
  } catch (ex) {
    this._logger.error("ERROR updating auth states for " + CLASS_SHORT_NAME
                       + ": " + ex);
  }

  profiler.profileEventEnd(evtID, "");
  gSvc = this;

  var ws = FlockSvcUtils.flockIWebService;
  ws.addDefaultMethod(this, "getStringBundle");
  this._loginURL = "https://pownce.com/login/";
  ws.addDefaultMethod(this, "loginURL");
  ws.addDefaultMethod(this, "isHidden");

  var lws = FlockSvcUtils.flockIManageableWebService;
  lws.addDefaultMethod(this, "docRepresentsSuccessfulLogin");
  ws.addDefaultMethod(this, "getAccount");
  ws.addDefaultMethod(this, "getAccounts");
  lws.addDefaultMethod(this, "getAccountIDFromDocument");
  ws.addDefaultMethod(this, "getAuthenticatedAccount");
  lws.addDefaultMethod(this, "getCredentialsFromForm");
  lws.addDefaultMethod(this, "getSessionValue");
  lws.addDefaultMethod(this, "ownsDocument");
  lws.addDefaultMethod(this, "ownsLoginForm");
  ws.addDefaultMethod(this, "removeAccount");

  var rcdh = FlockSvcUtils.flockIRichContentDropHandler;
  rcdh.addDefaultMethod(this, "_handleTextareaDrop");
}


/*************************************************************************
 * PownceService: XPCOM Component Creation
 *************************************************************************/

PownceService.prototype = new FlockXPCOMUtils.genericComponent(
  CLASS_NAME,
  CLASS_ID,
  CONTRACT_ID,
  PownceService,
  CI.nsIClassInfo.SINGLETON,
  [
    CI.nsIObserver,
    CI.flockIWebService,
    CI.flockIManageableWebService,
    CI.flockIPollingService,
    CI.flockISocialWebService,
    CI.flockIRichContentDropHandler
  ]
);

// FlockXPCOMUtils.genericModule() categories
PownceService.prototype._xpcom_categories = [
  { category: "wsm-startup" },
  { category: "flockWebService", entry: CLASS_SHORT_NAME },
  { category: "flockRichContentHandler", entry: CLASS_SHORT_NAME }
];


/*************************************************************************
 * PownceService: Private Data and Functions
 *************************************************************************/
PownceService.prototype._lightPeopleIcon =
function PownceService__lightPeopleIcon() {
  this._logger.debug("._lightPeopleIcon()");
  this._obs.notifyObservers(null, "new-people-notification", null);
};


PownceService.prototype._addPerson =
function PownceService__addPerson(aAccount, aPerson) {
/* Can't get last profile update time from Pownce
  // Make sure null update times aren't missed
  var lastProfileUpdate = 0;
  if (aPerson.profile_update_time) {
    lastProfileUpdate = parseInt(aPerson.profile_update_time, 10);
  }
*/
  // Make sure null status times aren't missed
  var statusTime = 0;
  var statusMessage = "";
  if (aPerson.note && aPerson.note.body && aPerson.note.timestamp) {
    statusTime = parseInt(aPerson.note.timestamp, 10);
    statusMessage = aPerson.note.body;
  }

  var lastUpdate = 0;
  var lastUpdateType;

  //if (statusTime > lastProfileUpdate) {
    lastUpdate = statusTime;
    lastUpdateType = "status";
  /*
  } else {
    lastUpdate = lastProfileUpdate;
    lastUpdateType = "profile";
  }
  */
  var avatarUrl = null;
  if (aPerson.profile_photo_urls &&
      aPerson.profile_photo_urls.medium_photo_url &&
      !this._hasDefaultAvatar(aPerson.profile_photo_urls.medium_photo_url))
  {
    avatarUrl = aPerson.profile_photo_urls.medium_photo_url;
  }

  var person = CC["@flock.com/person;1"]
               .createInstance(CI.flockIPerson);
  person.accountId = aPerson.id;
  person.screenName = aPerson.username;
  person.lastUpdate = lastUpdate;
  person.lastUpdateType = lastUpdateType;
  person.name = aPerson.short_name;
  person.avatar = avatarUrl;
  person.statusMessage = statusMessage;

  this._logger.debug("Adding person: '"
                     + person.accountId + "' '"
                     + person.screenName + "' '"
                     + person.statusMessage + "'");

  this.ppUtils.addPerson(aAccount, person);
};


/**
 * Helper function to determine if the user has customized their avatar based
 * on the passed in URL.
 * @param  aUrl  A string containing the contents of the Pownce user's
 *               "profile_photo_urls.medium_photo_url" property.
 * @return  true if the user is still using the default avatar, else false
 */
PownceService.prototype._hasDefaultAvatar =
function PownceService__hasDefaultAvatar(aUrl) {
  this._logger.debug("_hasDefaultAvatar(" + aUrl + ")");
  var defaultUrl = this._WebDetective
                       .getString(CLASS_SHORT_NAME, "noAvatar", "");
  return (aUrl.indexOf(defaultUrl) != -1);
};


/**************************************************************************
 * PownceService: nsIObserver Implementation
 **************************************************************************/

PownceService.prototype.observe =
function PownceService_observe(aSubject, aTopic, aState) {
  this._logger.info(".observe(..., '" + aTopic + "', '" + aState + "')");
  switch (aTopic) {
    case "xpcom-shutdown":
      this._obs.removeObserver(this, "xpcom-shutdown");
      break;
  }
};


/*************************************************************************
 * PownceService: flockIWebService Implementation
 *************************************************************************/

// readonly attribute AString urn;
PownceService.prototype.urn = "urn:" + CLASS_SHORT_NAME + ":service";

// readonly attribute AString title;
PownceService.prototype.title = CLASS_TITLE;

// readonly attribute AString icon;
PownceService.prototype.icon = FAVICON;

// readonly attribute AString url;
PownceService.prototype.url = POWNCE_URL;
PownceService.prototype.redirUrl = POWNCE_URL;

// readonly attribute AString contractId;
// ALMOST duplicated by nsIClassInfo::contractID
// with different case: contractId != contractID
PownceService.prototype.contractId = CONTRACT_ID;

// readonly attribute AString shortName;
PownceService.prototype.shortName = CLASS_SHORT_NAME;


/*************************************************************************
 * PownceService: flockIManageableWebService Implementation
 *************************************************************************/

// readonly attribute AString domains;
PownceService.prototype.__defineGetter__("domains",
function PownceService_getdomains() {
  this._logger.debug("Getting property: domains");
  return this._WebDetective
             .getString(CLASS_SHORT_NAME, "domains",
                        "pownce.com");
});

// readonly attribute boolean needPassword;
PownceService.prototype.needPassword = true;

/**
 * @see flockIManageableWebService#addAccount
 */
PownceService.prototype.addAccount =
function PownceService_addAccount(aAccountId, aIsTransient, aListener) {
  this._logger.debug(".addAccount('" + aAccountId + "', "
                                     + aIsTransient + ", ...)");

  if (!aAccountId) {
    if (aListener) {
      var error = CC[FLOCK_ERROR_CONTRACTID].createInstance(CI.flockIError);
      // XXX See bug 10749 - flockIError cleanup
      error.errorCode = 9999;
      error.errorString = "No Account ID provided";
      aListener.onError(this, "addAccount", error);
    }
    return;
  }

  // XXX TODO: Scrape user's short name (ex: "Matthew W.") and use that for
  //           the coop.Account.name.
  var pw = this._acUtils.getPassword(this.urn + ":" + aAccountId);
  var name = (pw) ? pw.user : aAccountId;
  var url = this._WebDetective.getString(CLASS_SHORT_NAME, "userprofile", null)
                .replace("%accountid%", aAccountId);
  var accountUrn = this._acUtils.createAccount(this,
                                               aAccountId,
                                               name,
                                               url,
                                               aIsTransient);

  // Add custom parameters to be used
  var customParams = FlockSvcUtils.newResults();
  customParams.setPropertyAsInt32("friendRequests", 0);
  customParams.setPropertyAsInt32("maxUploadSize", 0);
  this._acUtils.addCustomParamsToAccount(customParams, accountUrn);

  // Instantiate account component
  var account = this.getAccount(accountUrn);
  // Is this service pollable by flockIPollerService?
  account.setParam("isPollable", true);
  account.setParam("refreshInterval", REFRESH_INTERVAL);
  if (aListener) {
    aListener.onSuccess(account, "addAccount");
  }
  return account;
};

// DEFAULT: boolean docRepresentsSuccessfulLogin(in nsIDOMHTMLDocument aDocument);
// DEFAULT: flockIWebServiceAccount getAccount(in AString aAccountURN);
// DEFAULT: AString getAccountIDFromDocument(in nsIDOMHTMLDocument aDocument);
// DEFAULT: nsISimpleEnumerator getAccounts();
// DEFAULT: flockIWebServiceAccount getAuthenticatedAccount();
// DEFAULT: nsIPassword getCredentialsFromForm(in nsIDOMHTMLFormElement aForm);
// DEFAULT: AString getSessionValue();

/**
 * @see flockIManageableWebService#logout
 */
PownceService.prototype.logout =
function PownceService_logout() {
  this._logger.debug(".logout()");
  var cookies = this._WebDetective.getSessionCookies(this.shortName);
  if (cookies) {
    this._acUtils.removeCookies(cookies);
  }

  gApi.deauthenticate();
};

// DEFAULT: boolean ownsDocument(in nsIDOMHTMLDocument aDocument);
// DEFAULT: boolean ownsLoginForm(in nsIDOMHTMLFormElement aForm);

/**
 * @see flockIManageableWebService#removeAccount
 */
PownceService.prototype.removeAccount =
function PownceService_removeAccount(aUrn) {
  this._logger.debug(".removeAccount(" + aUrn + ")");
  this._acUtils.removeAccount(aUrn);
};

// DEFAULT: flockIWebServiceAccount getAccount(in AString aUrn);
// DEFAULT: nsISimpleEnumerator getAccounts();


/**
 * void updateAccountStatusFromDocument(in nsIDOMHTMLDocument aDocument,
 *                                      in AString aAcctUrn,
 *                                      in flockIListener aAuthListener);
 * @see flockIManageableWebService#updateAccountStatusFromDocument
 */
PownceService.prototype.updateAccountStatusFromDocument =
function PownceService_updateAccountStatusFromDocument(aDocument,
                                                       aAcctUrn,
                                                       aAuthListener)
{
  this._logger.info(".updateAccountStatusFromDocument('" + aDocument.URL + "', '"
                                                         + aAcctUrn + "', ...)");
  if (aAcctUrn) {
    this._logger.debug("We're logged in!");
    var account = this.getAccount(aAcctUrn);
    var newlyAuthenticated = !account.isAuthenticated();
    this._acUtils.ensureOnlyAuthenticatedAccount(aAcctUrn);
    if (newlyAuthenticated) {
      gApi.authenticate(null, null);
      if (aAuthListener) {
        aAuthListener.onSuccess(account, null);
      }
    }
  } else if (this._WebDetective
                 .detect(this.shortName, "loggedout", aDocument, null))
  {
    this._logger.debug("We're logged out!");
    this._acUtils.markAllAccountsAsLoggedOut(CONTRACT_ID);
    gApi.deauthenticate();
  } else {
    this._logger.debug("We don't match 'loggedout' or 'logged in'");
  }
};


/*************************************************************************
 * PownceService: flockIPollingService Implementation
 *************************************************************************/
/**
 * @see flockIPollingService#refresh
 */
PownceService.prototype.refresh =
function PownceService_refresh(aUrn, aPollingListener) {
  this._logger.debug(".refresh('" + aUrn + "', ...)");
  var inst = this;
  var account = this.getAccount(aUrn)
                    .QueryInterface(CI.flockISocialWebServiceAccount);

  // If the user is not logged in, return a success without attempting to
  // refresh anything.
  if (!account.isAuthenticated()) {
    this._logger.debug(".refresh: Account is NOT authenticated. Short circuiting.");
    aPollingListener.onResult();
    return;
  }
  this._logger.debug(".refresh: Account is authenticated. Continuing.");

  var pplRefreshListener = CC["@flock.com/people-refresh-listener;1"]
                           .createInstance(CI.flockIPeopleRefreshListener);
  pplRefreshListener.init(3, aUrn, aPollingListener, SHORT_INTERVAL);

  // Example JSON from Pownce API v2.0
  // {
  //   "username": "lilmatt",
  //   "permalink": "http:\/\/pownce.com\/lilmatt\/",
  //   "short_name": "Matthew W.",
  //   "country": "United States",
  //   "age": 33,
  //   "first_name": "Matthew",
  //   "blurb": "lilmatt == (VW && Mozilla && Apple)",
  //   "gender": "Male",
  //   "profile_photo_urls": {
  //     "smedium_photo_url": "http:\/\/pownce.com\/profile_photos\/l\/i\/l\/lilmatt\/141242_smedium.jpg",
  //     "small_photo_url": "http:\/\/pownce.com\/profile_photos\/l\/i\/l\/lilmatt\/141242_small.jpg",
  //     "tiny_photo_url": "http:\/\/pownce.com\/profile_photos\/l\/i\/l\/lilmatt\/141242_tiny.jpg",
  //     "medium_photo_url": "http:\/\/pownce.com\/profile_photos\/l\/i\/l\/lilmatt\/141242_medium.jpg",
  //     "large_photo_url": "http:\/\/pownce.com\/profile_photos\/l\/i\/l\/lilmatt\/141242_large.jpg"
  //   },
  //   "id": 141242,
  //   "is_pro": 0,
  //   "max_upload_mb": 0,
  //   "location": "Ithaca, NY"
  // }

  // This listener handles getting the account owner's information.
  var userShowApiListener = {
    onSuccess: function userShow_onSuccess(aResult) {
      if (aResult.short_name) {
        inst._logger.debug("userShow: name: " + aResult.short_name);
        account.setParam("name", aResult.short_name);
      }

      if (aResult.username) {
        inst._logger.debug("userShow: screen_name: " + aResult.username);
        account.setParam("screenName", aResult.username);
      }

      if (aResult.is_pro) {
        inst._logger.debug("userShow: is_pro: " + aResult.is_pro);
        account.setParam("isPro", aResult.is_pro);
      }

      if (aResult.max_upload_mb) {
        inst._logger.debug("userShow: max_upload_mb: "
                           + aResult.max_upload_mb);
        account.setCustomParam("maxUploadSize", aResult.max_upload_mb);
      }

      if (aResult.permalink) {
        inst._logger.debug("userShow: URL: " + aResult.permalink);
        account.setParam("URL", aResult.permalink);
      }

      if (aResult.profile_photo_urls &&
          aResult.profile_photo_urls.medium_photo_url)
      {
        var photoUrl = aResult.profile_photo_urls.medium_photo_url;
        inst._logger.debug("userShow: avatar: " + photoUrl);

        // If avatar returned is the default image, set coop.Account.avatar
        // to null and let the people sidebar code set the Flock common image.
        if (inst._hasDefaultAvatar(photoUrl)) {
          inst._logger.debug("No avatar for account. Setting to null.");
          account.setParam("avatar", null);
        } else {
          account.setParam("avatar", photoUrl);
        }
      }

      if (aResult.friend_request_count) {
        var friendRequests = aResult.friend_request_count;
        inst._logger.debug("userShow: friendRequests: " + friendRequests);

        // Light people icon if necessary
        var existingFriendRequests = account.getCustomParam("friendRequests");
        if (existingFriendRequests < friendRequests) {
          inst._lightPeopleIcon();
        }

        account.setCustomParam("friendRequests", friendRequests);
      }

      pplRefreshListener.onSuccess(null, "userShowApiListener");
    },
    onError: function userShow_onError(aError) {
      inst._logger.error("userShowApiListener onError");
      pplRefreshListener.onError(null, "userShowApiListener", aError);
    }
  };

  // This handles getting the user's latest public note (a.k.a. "status").
  var getMyLastNoteListener = {
    onSuccess: function getMyLastNote_onSuccess(aResult) {
      inst._logger.debug("getMyLastNoteListener onSuccess");

      var result = aResult.notes[0];
      if (result && result.body && result.timestamp) {
        inst._logger.debug("getMyLastNoteListener: result.body: "
                           + result.body);
        account.setParam("statusMessage", result.body);
        account.setParam("lastStatusMessageUpdateDate", result.timestamp);
      } else {
        inst._logger.debug("getMyLastNoteListener: No result to parse!");
        account.setParam("statusMessage", null);
        account.setParam("lastStatusMessageUpdateDate", 0);
      }
      pplRefreshListener.onSuccess(null, "getMyLastNoteListener");
    },
    onError: function getMyLastNote_onError(aError) {
      inst._logger.error("getMyLastNoteListener onError: " + aError);
      pplRefreshListener.onError(null, "getMyLastNoteListener", aError);
    }
  };

  // This listener handles getting the user's friends' information.
  var friendsApiListener = {
    onSuccess: function friends_onSuccess(aResult) {
      inst._logger.debug("friendsApiListener onSuccess");

      // Now that we have all the results we need, update the RDF.

      // ADD or update existing people
      for (var friend in aResult) {
        inst._addPerson(account, aResult[friend]);
      }

      // REMOVE locally people removed on the server
      var localFriends = account.enumerateFriends();
      while (localFriends.hasMoreElements()) {
        var localFriend = localFriends.getNext();
        if (!aResult[localFriend.accountId]) {
          inst._logger.info("Friend " + localFriend.accountId
                            + " has been deleted on the server");
          inst.ppUtils.removePerson(account, localFriend);
        }
      }
      pplRefreshListener.onSuccess(null, "friendsApiListener");
    },
    onError: function friends_onError(aError) {
      inst._logger.error("friendsApiListener onError");
      pplRefreshListener.onError(null, "friendsApiListener", aError);
    }
  };

  var uid = account.getParam("accountId");
  gApi.userShow(uid, userShowApiListener);
  gApi.getLastPublicNoteByUser(uid, getMyLastNoteListener);
  gApi.getFriendsStatus(uid, friendsApiListener);

  // Prevent JS strict warnings.  We used a return earlier to short-circuit
  // this in the case of being unauthenticated, so we need to return here also.
  return;
};


/*************************************************************************
 * PownceService: flockISocialWebService implementation
 *************************************************************************/

// readonly attribute long maxStatusLength;
PownceService.prototype.maxStatusLength = POWNCE_MAX_STATUS_LENGTH;

// void markAllMediaSeen(in AString aPersonUrn);
PownceService.prototype.markAllMediaSeen =
function PownceService_markAllMediaSeen(aPersonUrn) {
  this._logger.debug(".markAllMediaSeen('" + aPersonUrn + "')");

  // Pownce has no media, so this does nothing and shouldn't be called.
  throw (NS_ERROR_NOT_IMPLEMENTED);
};


/**************************************************************************
 * PownceService: flockIRichContentDropHandler Implementation
 **************************************************************************/

PownceService.prototype.handleDrop =
function PownceService_handleDrop(aFlavours, aTextarea) {
  this._logger.debug(".handleDrop()");

  var inst = this;
  var dropCallback = function handleDrop_dropCallback(aFlav) {
    var dataObj = {};
    var len = {};
    aFlavours.getTransferData(aFlav, dataObj, len);

    var text = dataObj.value.QueryInterface(CI.nsISupportsString).data;
    var textParts = text.split(": ");
    var url = (textParts.length == 2) ? textParts[1] : text;

    // Find position
    var caretPos = aTextarea.selectionEnd;
    var currentValue = aTextarea.value;

    // Add a trailing space so that we don't mangle the url
    var nextChar = currentValue.charAt(caretPos);
    var trailingSpace = ((nextChar == "") ||
                         (nextChar == " ") ||
                         (nextChar == "\n"))
                      ? ""
                      : " ";

    // Put it all together to drop the text into the selection.
    // Note: no breadcrumb due to Pownce length constraint.
    aTextarea.value = currentValue.substring(0, caretPos)
                    + url
                    + trailingSpace
                    + currentValue.substring(caretPos);
  };

  return this._handleTextareaDrop(CLASS_SHORT_NAME, this.domains,
                                  aTextarea, dropCallback);
};


/*************************************************************************
 * Component: PownceAccount
 *************************************************************************/
function PownceAccount() {
  FlockSvcUtils.getLogger(this);
  this._logger.init("PownceAccount");

  FlockSvcUtils.getACUtils(this);
  FlockSvcUtils.getCoop(this);
  this.ppUtils = CC["@flock.com/people-utils;1"]
                  .getService(CI.flockIPeopleUtils);
  // XXX: I should be able to use FlockSvcUtils.getWD() here, but it can
  //      only be called by the service.
  this._WebDetective = CC["@flock.com/web-detective;1"]
                       .getService(CI.flockIWebDetective);

  var wsa = FlockSvcUtils.flockIWebServiceAccount;
  wsa.addDefaultMethod(this, "getAllCustomParams");
  wsa.addDefaultMethod(this, "getCustomParam");
  wsa.addDefaultMethod(this, "getParam");
  wsa.addDefaultMethod(this, "getService");
  wsa.addDefaultMethod(this, "isAuthenticated");
  wsa.addDefaultMethod(this, "login");
  wsa.addDefaultMethod(this, "logout");
  wsa.addDefaultMethod(this, "setCustomParam");
  wsa.addDefaultMethod(this, "setParam");

  var sa = FlockSvcUtils.flockISocialWebServiceAccount;
  sa.addDefaultMethod(this, "enumerateFriends");
  //sa.addDefaultMethod(this, "formatFriendActivityForDisplay");
  sa.addDefaultMethod(this, "getFriendCount");
  sa.addDefaultMethod(this, "getFormattedFriendUpdateType");

  var sbs = CC["@mozilla.org/intl/stringbundle;1"]
            .getService(CI.nsIStringBundleService);
  this._bundle = sbs.createBundle(POWNCE_PROPERTIES);
}


/*************************************************************************
 * PownceAccount: XPCOM Component Creation
 *************************************************************************/

// Use genericComponent() for the goodness it provides (QI, nsIClassInfo, etc),
// but do NOT add this component to the list of constructors passed to
// FlockXPCOMUtils.genericModule().
PownceAccount.prototype = new FlockXPCOMUtils.genericComponent(
  CLASS_NAME + " Account",
  "",
  "",
  PownceAccount,
  0,
  [
    CI.flockIWebServiceAccount,
    CI.flockISocialWebServiceAccount
  ]
  );


/*************************************************************************
 * PownceAccount: flockIWebServiceAccount Implementation
 *************************************************************************/

// readonly attribute AString urn;
PownceAccount.prototype.urn = "";

// DEFAULT: nsIPropertyBag getAllCustomParams();
// DEFAULT: nsIVariant getCustomParam(in AString aParamName);
// DEFAULT: nsIVariant getParam(in AString aParamName);
// DEFAULT: flockIManageableWebService getService();
// DEFAULT: boolean isAuthenticated();

/**
 * void keep();
 * @see flockIManageableWebService#keep
 */
PownceAccount.prototype.keep =
function PownceAccount_keep() {
  this._logger.debug(".keep()");
  this.setParam("isTransient", false);
  var urn = this.urn.replace("account:", "service:").replace("flock:", "");
  this._acUtils.makeTempPasswordPermanent(urn);
};

// DEFAULT: void login(in flockIListener aListener);
// DEFAULT: void logout(in flockIListener aListener);
// DEFAULT: void setCustomParam(in AString aParamName, in nsIVariant aValue);
// DEFAULT: void setParam(in AString aParamName, in nsIVariant aValue);


// void activate(in flockIListener aListener);
PownceAccount.prototype.activate =
function PownceAccount_activate(aListener) {
  this._logger.debug(".activate()");
  this.setParam("isAuthenticated", true);
  this.setParam("nextRefresh", new Date(0));
  this.setParam("isPollable", true);

  if (aListener) {
    aListener.onSuccess(this, "activate");
  }
};

// DEFAULT: void deactivate(in flockIListener aListener);
// DEFAULT: void login(in flockIListener aListener);
// DEFAULT: void logout(in flockIListener aListener);


/*************************************************************************
 * PownceAccount: flockISocialWebServiceAccount Implementation
 *************************************************************************/

// readonly attribute boolean hasFriendActions;
PownceAccount.prototype.hasFriendActions = true;

// readonly attribute boolean isMyMediaFavoritesSupported;
PownceAccount.prototype.isMyMediaFavoritesSupported = false;

// readonly attribute boolean isPostLinkSupported;
PownceAccount.prototype.isPostLinkSupported = true;

// readonly attribute boolean isStatusEditable;
PownceAccount.prototype.isStatusEditable = true;

// readonly attribute boolean isMeStatusSupported;
PownceAccount.prototype.isMeStatusSupported = true;

// readonly attribute boolean isFriendStatusSupported;
PownceAccount.prototype.isFriendStatusSupported = true;

// DEFAULT: nsISimpleEnumerator enumerateFriends();

// AString formatFriendActivityForDisplay(in AString aFriendActivityUrn);
PownceAccount.prototype.formatFriendActivityForDisplay =
function PownceAccount_formatFriendActivityForDisplay(aFriendActivityUrn) {
  this._logger.debug(".formatFriendActivityForDisplay()");

  var friendActivity = this._coop.get(aFriendActivityUrn);
  var prefix = "flock." + CLASS_SHORT_NAME + ".";

  var sb = this._bundle;
  var out = "";

  switch (friendActivity.updateType) {
    case "profile":
      out = sb.GetStringFromName(prefix + "lastUpdateTypePretty.profile");
      break;

    case "status":
      if (friendActivity.updateValue == "") {
        out = sb.GetStringFromName(prefix + "lastUpdateTypePretty.statusCleared");
      } else {
        out = sb.formatStringFromName(prefix + "lastUpdateTypePretty.statusUpdated",
                                     [friendActivity.updateValue], 1);
      }
      break;
  }

  return (flockXMLDecode(out));
};

/**
 * AString formatStatusForDisplay(in AString aStatusMessage);
 * @see flockISocialWebServiceAccount#formatStatusForDisplay
 */
PownceAccount.prototype.formatStatusForDisplay =
function PownceAccount_formatStatusForDisplay(aStatusMessage) {
  this._logger.debug(".formatStatusForDisplay('" + aStatusMessage + "')");

  var message = (aStatusMessage) ? aStatusMessage : "";

  // Pownce uses "\r\n" to represent <br/>
  message = message.replace(/\r\n/g, " ");
  return flockXMLDecode(message);
};

/**
 * void setStatus(in AString aStatusMessage, in flockIListener aListener);
 * @see flockISocialWebServiceAccount#setStatus
 */
PownceAccount.prototype.setStatus =
function PownceAccount_setStatus(aStatusMessage, aListener) {
  this._logger.debug(".setStatus('" + aStatusMessage + "', ...)");

  var inst = this;
  var statusApiListener = {
    onSuccess: function L_onSuccess(aResult) {
      // If the API call succeeded, also set the account status.
      var message = (aResult && aResult.body) ? aResult.body : aStatusMessage;
      inst.setParam("statusMessage", message);
      if (aListener) {
        aListener.onSuccess(aResult, "setStatus");
      }
    },
    onError: function L_onError(aError) {
      if (aListener) {
        aListener.onError(null, "setStatus", aError);
      }
    }
  };
  gApi.setStatus(aStatusMessage, statusApiListener);
};

/**
 * AString getEditableStatus();
 * @see flockISocialWebServiceAccount#getEditableStatus
 */
PownceAccount.prototype.getEditableStatus =
function PownceAccount_getEditableStatus() {
  this._logger.debug(".getEditableStatus()");
  var message = this.getParam("statusMessage");
  message = (message) ? flockXMLDecode(message) : "";
  return this.formatStatusForDisplay(message);
};

/**
 * AString getFriendActions(in AString aFriendUrn);
 * @see flockISocialWebServiceAccount#getFriendActions
 */
PownceAccount.prototype.getFriendActions =
function PownceAccount_getFriendActions(aFriendUrn) {
  this._logger.debug(".getFriendActions('" + aFriendUrn + "')");

  var actionNames = ["friendMessage",
                     "friendShareLink",
                     "friendViewProfile"];

  var actions = [];
  var friend = this.ppUtils.getPerson(aFriendUrn);
  if (friend) {
    var coopAccount = this._coop.get(this.urn);
    for each (var action in actionNames) {
      actions.push({
        label: this._bundle.GetStringFromName("flock."
                                              + CLASS_SHORT_NAME
                                              + ".actions." + action),
        class: action,
        spec: this._WebDetective.getString(CLASS_SHORT_NAME, action, "")
                  .replace("%accountid%", coopAccount.accountId)
                  .replace("%friendid%", friend.accountId)
                  .replace("%friendScreenName%", friend.screenName)
      });
    }
  }

  return JSON.toString(actions);
};

// DEFAULT: long getFriendCount();

/**
 * AString getMeNotifications();
 * @see flockISocialWebServiceAccount#getMeNotifications
 */
PownceAccount.prototype.getMeNotifications =
function PownceAccount_getMeNotifications() {
  this._logger.debug(".getMeNotifications()");

  var noties = [];
  var inst = this;
  function addNotie(aType, aCount) {
    var stringName = "flock." + CLASS_SHORT_NAME + ".noties." + aType + "."
                   + ((parseInt(aCount) <= 0) ? "none" : "some");

    inst._logger.debug("aType: " + aType
                       + " aCount: " + aCount
                       + " name: " + stringName);
    noties.push({
      class: aType,
      tooltip: inst._bundle.GetStringFromName(stringName),
      count: aCount,
      URL: inst._WebDetective.getString(CLASS_SHORT_NAME, aType + "_URL", "")
                             .replace("%accountid%",
                                      inst.getParam("accountId"))
    });
  }
  var coopAccount = this._coop.get(this.urn);
  //addNotie("meMessages", coopAccount.accountMessages);
  addNotie("meFriendRequests", this.getCustomParam("friendRequests"));

  return JSON.toString(noties);
};

/**
 * AString getPostLinkAction(in nsITransferable aTransferable);
 * @see flockISocialWebServiceAccount#getPostLinkAction
 */
PownceAccount.prototype.getPostLinkAction =
function PownceAccount_getPostLinkAction(aTransferable) {
  var url = "";

  if (aTransferable) {
    // Something was dropped onto the "Post Link" button: get the URL from the
    // transferable
    var flavours = ["text/x-flock-media",
                    "text/x-moz-url",
                    "text/unicode"];

    var message = CC[FLOCK_RICH_DND_CONTRACTID]
                  .getService(CI.flockIRichDNDService)
                  .getMessageFromTransferable(aTransferable,
                                              flavours.length,
                                              flavours);

    url = message.body;
  } else {
    // The "Post Link" button was clicked: get the current tab's URL
    var win = CC["@mozilla.org/appshell/window-mediator;1"]
              .getService(CI.nsIWindowMediator)
              .getMostRecentWindow("navigator:browser");
    if (win) {
      url = win.gBrowser.currentURI.spec;
    }
  }

  var action = "";
  if (url) {
    action = this._WebDetective
                 .getString(CLASS_SHORT_NAME, "shareAction_postLink", "")
                 .replace("%url%", encodeURIComponent(url));
  }

  return action;
};

/**
 * AString getProfileURLForFriend(in AString aFriendUrn);
 * @see flockISocialWebServiceAccount#getProfileURLForFriend
 */
PownceAccount.prototype.getProfileURLForFriend =
function PownceAccount_getProfileURLForFriend(aFriendUrn) {
  this._logger.debug(".getProfileURLForFriend('" + aFriendUrn + "')");

  var url = "";
  var friend = this.ppUtils.getPerson(aFriendUrn);
  var screenName = friend.screenName;
  if (screenName) {
    this._logger.debug(".getProfileURLForFriend screenName: " + screenName);
    url = this._WebDetective.getString(CLASS_SHORT_NAME, "friendProfile", "")
                            .replace("%screenName%", screenName);
  }

  return url;
};

/**
 * AString getSharingAction(in AString aFriendUrn,
 *                          in nsITransferable aTransferable);
 * @see flockISocialWebServiceAccount#getSharingAction
 */
PownceAccount.prototype.getSharingAction =
function PownceAccount_getSharingAction(aFriendUrn, aTransferable) {
  this._logger.debug(".getSharingAction('" + aFriendUrn + "')");
  var sharingAction = "";
  var coopFriend = this._coop.get(aFriendUrn);
  if (!coopFriend) {
    return sharingAction;
  }

  var flavours = ["text/x-flock-media",
                  "text/x-moz-url",
                  "text/unicode"];

  var message = CC[FLOCK_RICH_DND_CONTRACTID]
                .getService(CI.flockIRichDNDService)
                .getMessageFromTransferable(aTransferable,
                                            flavours.length,
                                            flavours);
  if (!message.body) {
    return sharingAction;
  }

  sharingAction = this._WebDetective
                      .getString(CLASS_SHORT_NAME, "friendShareText", "")
                      .replace("%friendid%", coopFriend.accountId)
                      .replace("%body%", encodeURIComponent(message.body));

  this._logger.debug(".getSharingAction(): " + sharingAction);
  return sharingAction;
};

/**
 * void markAllMeNotificationsSeen(in AString aType);
 * @see flockISocialWebServiceAccount#markAllMediaNotificationsSeen
 */
PownceAccount.prototype.markAllMeNotificationsSeen =
function PownceAccount_markAllMeNotificationsSeen(aType) {
  this._logger.debug(".markAllMeNotificationsSeen('" + aType + "')");
  switch (aType) {
    case "meMessages":
      this.setParam("accountMessages", 0);
      break;
    case "meFriendRequests":
      this.setCustomParam("friendRequests", 0);
      break;
    default:
      break;
  }
};


/*************************************************************************
 * XPCOM Support - Module Construction
 *************************************************************************/

// Create array of components.
var componentsArray = [PownceService];

// Generate a module for XPCOM to find.
var NSGetModule = FlockXPCOMUtils.generateNSGetModule(MODULE_NAME,
                                                      componentsArray);
